@@ -113,7 +113,7 @@ public:
 	typedef TimeVectorsSuzukiTrotter<TargetParamsType,ModelType,WaveFunctionTransfType,
 	LanczosSolverType,VectorWithOffsetType> TimeVectorsSuzukiTrotterType;
 	typedef typename ModelType::InputValidatorType InputValidatorType;
-	typedef typename PsimagLite::Vector<VectorWithOffsetType>::Type VectorVectorWithOffsetType;
+	typedef typename PsimagLite::Vector<VectorWithOffsetType*>::Type VectorVectorWithOffsetType;
 	typedef typename PsimagLite::Vector<SizeType>::Type VectorSizeType;
 	typedef typename PsimagLite::Vector<RealType>::Type VectorRealType;
 	typedef ApplyOperatorLocal<LeftRightSuperType,VectorWithOffsetType> ApplyOperatorType;
@@ -135,12 +135,13 @@ public:
 	    : progress_("ApplyOperatorExpression"),
 	      targetHelper_(targetHelper),
 	      E0_(0.0),
-	      time_(0.0),
-	      currentTimeStep_(0),
 	      indexNoAdvance_(indexNoAdvance),
 	      applyOpLocal_(targetHelper.lrs(), targetHelper.withLegacyBugs()),
 	      targetVectors_(0),
-	      timeVectorsBase_(0),
+	      timeVectorsBase_(new TimeVectorsBaseType(targetHelper_.model(),
+	                                               targetHelper_.lrs(),
+	                                               targetHelper_.wft(),
+	                                               "base")),
 	      wftHelper_(targetHelper.model(), targetHelper.lrs(), targetHelper.wft()),
 	      multiSiteExprHelper_(targetHelper_.model().superGeometry().numberOfSites() - 2),
 	      correlationsSkel_(multiSiteExprHelper_, targetHelper.model(), false)
@@ -154,6 +155,12 @@ public:
 		delete timeVectorsBase_;
 		timeVectorsBase_ = 0;
 		clearPsi();
+
+		const SizeType n = targetVectors_.size();
+		for (SizeType i = 0; i < n; ++i) {
+			delete targetVectors_[i];
+			targetVectors_[i] = nullptr;
+		}
 	}
 
 	virtual void initPsi(SizeType nsectors, SizeType nexcited)
@@ -292,20 +299,11 @@ public:
 		stage_[ind] = x;
 	}
 
-	const RealType& energy() const
-	{
-		return E0_;
-	}
+	const RealType& energy() const { return E0_; }
 
-	const ApplyOperatorType& applyOpLocal() const
-	{
-		return applyOpLocal_;
-	}
+	const ApplyOperatorType& applyOpLocal() const { return applyOpLocal_; }
 
-	const VectorVectorVectorWithOffsetType& psiConst() const
-	{
-		return psi_;
-	}
+	const VectorVectorVectorWithOffsetType& psiConst() const { return psi_; }
 
 	void setOnlyOnePsi(const VectorWithOffsetType& v)
 	{
@@ -387,27 +385,28 @@ public:
 		}
 	}
 
-	const VectorVectorWithOffsetType& targetVectors() const
-	{
-		return targetVectors_;
-	}
+	const SizeType tvs() const { return targetVectors_.size(); }
 
-	VectorWithOffsetType& targetVectors(SizeType i) // <--- FIXME
+	VectorWithOffsetType& targetVectorsNonConst(SizeType i) // <--- FIXME
 	{
 		assert(i < targetVectors_.size());
-		return targetVectors_[i];
+		assert(targetVectors_[i]);
+		return *targetVectors_[i];
 	}
 
 	const VectorWithOffsetType& targetVectors(SizeType i) const
 	{
 		assert(i < targetVectors_.size());
-		return targetVectors_[i];
+		assert(targetVectors_[i]);
+		return *targetVectors_[i];
 	}
 
-	SizeType createPvector(const VectorWithOffsetType& v)
+	SizeType createPvector(const VectorWithOffsetType& vv)
 	{
 		const SizeType n = targetVectors_.size();
+		VectorWithOffsetType* v = new VectorWithOffsetType();
 		targetVectors_.push_back(v);
+		*targetVectors_[n] = vv;
 		return n;
 	}
 
@@ -416,7 +415,7 @@ public:
 		if (ind >= targetVectors_.size())
 			err("AOE: destroyPvector\n");
 
-		targetVectors_[ind].clear();
+		targetVectors_[ind]->clear();
 	}
 
 	void trimVectors()
@@ -425,7 +424,7 @@ public:
 		SizeType x = 0;
 		for (SizeType ii = 0; ii < n; ++ii) {
 			const SizeType i = n - ii - 1;
-			if (targetVectors_[i].size() == 0) {
+			if (targetVectors_[i]->size() == 0) {
 				++x;
 			} else {
 				break;
@@ -438,8 +437,26 @@ public:
 
 	void targetVectorsResize(SizeType x)
 	{
-		if (x == 0) targetVectors_.clear();
-		else targetVectors_.resize(x);
+		const SizeType n = targetVectors_.size();
+		if (n == x) return;
+
+		if (n < x) {
+			const SizeType r = x - n;
+			for (SizeType i = 0; i < r; ++i) {
+				VectorWithOffsetType* v = new VectorWithOffsetType();
+				targetVectors_.push_back(v);
+			}
+		} else {
+			assert(n > x);
+			const SizeType r = n - x;
+			for (SizeType i = 0; i < r; ++i) {
+				const SizeType j = n - i - 1;
+				delete targetVectors_[j];
+				targetVectors_[j] = nullptr;
+			}
+
+			targetVectors_.resize(x);
+		}
 
 		PsimagLite::OstringStream msgg(std::cout.precision());
 		PsimagLite::OstringStream::OstringStreamType& msg = msgg();
@@ -448,9 +465,10 @@ public:
 	}
 
 	void initTimeVectors(const TargetParamsType& tstStruct,
-	                     const VectorRealType& times,
 	                     InputValidatorType& ioIn)
 	{
+		delete timeVectorsBase_;
+		timeVectorsBase_ = nullptr;
 		const LeftRightSuperType& lrs = targetHelper_.lrs();
 		const ModelType& model = targetHelper_.model();
 		const WaveFunctionTransfType& wft = targetHelper_.wft();
@@ -460,9 +478,7 @@ public:
 
 		switch (tstStruct.algorithm()) {
 		case TargetParamsType::AlgorithmEnum::KRYLOV:
-			timeVectorsBase_ = new TimeVectorsKrylovType(currentTimeStep_,
-			                                             tstStruct,
-			                                             times,
+			timeVectorsBase_ = new TimeVectorsKrylovType(tstStruct,
 			                                             targetVectors_,
 			                                             model,
 			                                             wft,
@@ -470,9 +486,7 @@ public:
 			                                             ioIn);
 			break;
 		case TargetParamsType::AlgorithmEnum::CHEBYSHEV:
-			timeVectorsBase_ = new TimeVectorsChebyshevType(currentTimeStep_,
-			                                                tstStruct,
-			                                                times,
+			timeVectorsBase_ = new TimeVectorsChebyshevType(tstStruct,
 			                                                targetVectors_,
 			                                                model,
 			                                                wft,
@@ -480,18 +494,14 @@ public:
 			                                                ioIn);
 			break;
 		case TargetParamsType::AlgorithmEnum::RUNGE_KUTTA:
-			timeVectorsBase_ = new TimeVectorsRungeKuttaType(currentTimeStep_,
-			                                                 tstStruct,
-			                                                 times,
+			timeVectorsBase_ = new TimeVectorsRungeKuttaType(tstStruct,
 			                                                 targetVectors_,
 			                                                 model,
 			                                                 wft,
 			                                                 lrs);
 			break;
 		case TargetParamsType::AlgorithmEnum::SUZUKI_TROTTER:
-			timeVectorsBase_ = new TimeVectorsSuzukiTrotterType(currentTimeStep_,
-			                                                    tstStruct,
-			                                                    times,
+			timeVectorsBase_ = new TimeVectorsSuzukiTrotterType(tstStruct,
 			                                                    targetVectors_,
 			                                                    model,
 			                                                    wft,
@@ -502,21 +512,10 @@ public:
 		}
 	}
 
-	RealType time() const
+	const TimeVectorsBaseType& timeVectors() const
 	{
-		return time_;
-	}
-
-	SizeType currentTimeStep() const { return currentTimeStep_; }
-
-	void setCurrentTimeStep(SizeType t)
-	{
-		currentTimeStep_ = t;
-	}
-
-	void setCurrentTime(RealType t)
-	{
-		time_ = t;
+		assert(timeVectorsBase_);
+		return *timeVectorsBase_;
 	}
 
 	void loadEnergy(PsimagLite::IoSelector::In& io,
@@ -543,7 +542,7 @@ public:
 
 	void multiplyTimeVector(SizeType i,RealType factor)
 	{
-		targetVectors_[i] = factor*targetVectors_[i];
+		*targetVectors_[i] = factor*(*targetVectors_[i]);
 	}
 
 	void calcTimeVectors(const PsimagLite::Vector<SizeType>::Type& indices,
@@ -559,9 +558,8 @@ public:
 		                                              allOperatorsApplied,
 		                                              wftAndAdvanceIfNeeded,
 		                                              block,
-		                                              isLastCall,
-		                                              time_);
-		if (!timeVectorsBase_)
+		                                              isLastCall);
+		if (timeVectorsBase_->isBase())
 			err("timeVectorsBase_ ptr not setup!?\n");
 
 		timeVectorsBase_->calcTimeVectors(indices,
@@ -626,8 +624,7 @@ public:
 
 	void wftSome(SizeType site, SizeType begin, SizeType end)
 	{
-		VectorVectorWithOffsetType& tvs = const_cast<VectorVectorWithOffsetType&>(targetVectors_);
-		wftHelper_.wftSome(tvs, site, begin, end);
+		wftHelper_.wftSome(targetVectors_, site, begin, end);
 	}
 
 	void wftOneVector(VectorWithOffsetType& phiNew,
@@ -652,9 +649,7 @@ public:
 
 	void timeHasAdvanced()
 	{
-		if (!timeVectorsBase_)
-			err("timeHasAdvanced(): timeVectorsBase_ ptr not setup!?\n");
-		timeVectorsBase_->timeHasAdvanced(time_);
+		timeVectorsBase_->timeHasAdvanced();
 	}
 
 	const ModelType& model() const { return targetHelper_.model(); }
@@ -747,9 +742,9 @@ private:
 		if (advanceEach > 0 && timesWithoutAdvancement_ >= advanceEach && !dontAdvance) {
 			stage_[i] = StageEnum::WFT_ADVANCE;
 			if (i == lastI) {
-				++currentTimeStep_;
+				timeVectorsBase_->advanceCurrentTimeStep();
 				timesWithoutAdvancement_ = 1;
-				if (timeVectorsBase_) timeVectorsBase_->timeHasAdvanced(time_);
+				timeVectorsBase_->timeHasAdvanced();
 			}
 		} else {
 			if (i == lastI &&
@@ -764,10 +759,11 @@ private:
 		        site==1)
 			firstSeeLeftCorner_ = true;
 
+		const RealType time = timeVectorsBase_->time();
 		PsimagLite::OstringStream msgg2(std::cout.precision());
 		PsimagLite::OstringStream::OstringStreamType& msg2 = msgg2();
 		msg2<<"Steps without advance: "<<timesWithoutAdvancement_;
-		msg2<<" site="<<site<<" currenTime="<<time();
+		msg2<<" site="<<site<<" currenTime="<<time;
 		if (timesWithoutAdvancement_ > 0) progress_.printline(msgg2, std::cout);
 
 		PsimagLite::OstringStream msgg(std::cout.precision());
@@ -823,8 +819,10 @@ private:
 			const SizeType advanceEach = tstStruct.advanceEach();
 			SizeType advance = indexNoAdvance_;
 
-			if (advanceEach > 0 && stage_[i] == StageEnum::WFT_ADVANCE && timeVectorsBase_) {
-				SizeType timeSteps = tstStruct.timeSteps();
+			if (advanceEach > 0 &&
+			        stage_[i] == StageEnum::WFT_ADVANCE &&
+			        !timeVectorsBase_->isBase()) {
+				SizeType timeSteps = tstStruct.times().size();
 				advance = (timeSteps > 0) ? timeSteps - 1 : 0;
 			}
 
@@ -835,7 +833,8 @@ private:
 				throw PsimagLite::RuntimeError(s);
 			}
 
-			const VectorWithOffsetType& src = targetVectors_[advance];
+			assert(targetVectors_[advance]);
+			const VectorWithOffsetType& src = *targetVectors_[advance];
 
 			if (src.size() == 0) {
 				PsimagLite::String s(__FILE__);
@@ -846,7 +845,7 @@ private:
 			if  (site == 0 || site == numberOfSites -1) {
 				// don't wft since we did it before
 				assert(advance < targetVectors_.size());
-				if (timeVectorsBase_ || advanceEach == 0)
+				if (!timeVectorsBase_->isBase() || advanceEach == 0)
 					phiNew = src;
 				return;
 			}
@@ -877,8 +876,6 @@ private:
 	const TargetHelperType& targetHelper_;
 	VectorStageEnumType stage_;
 	RealType E0_;
-	RealType time_;
-	SizeType currentTimeStep_;
 	SizeType indexNoAdvance_;
 	ApplyOperatorType applyOpLocal_;
 	VectorVectorVectorWithOffsetType psi_;
