@@ -87,6 +87,7 @@ DISCLOSED WOULD NOT INFRINGE PRIVATELY OWNED RIGHTS.
 #include "ProgramGlobals.h"
 #include "Io/IoSelector.h"
 #include "DiskOrMemoryStack.h"
+#include "FiniteLoop.h"
 
 namespace Dmrg {
 
@@ -96,6 +97,7 @@ class Checkpoint {
 public:
 
 	typedef WaveFunctionTransfType_ WaveFunctionTransfType;
+	typedef typename WaveFunctionTransfType::LeftRightSuperType LeftRightSuperType;
 	typedef typename ModelType::RealType  RealType;
 	typedef typename ModelType::ParametersType ParametersType;
 	typedef typename ModelType::BasisWithOperatorsType BasisWithOperatorsType;
@@ -113,6 +115,8 @@ public:
 	typedef PsimagLite::Vector<SizeType>::Type VectorSizeType;
 	typedef typename PsimagLite::Vector<RealType>::Type VectorRealType;
 	typedef typename PsimagLite::Vector<VectorRealType>::Type VectorVectorRealType;
+	using FiniteLoopType = FiniteLoop<RealType>;
+	using VectorFiniteLoopType = typename PsimagLite::Vector<FiniteLoopType>::Type;
 
 	Checkpoint(const ParametersType& parameters,
 	           InputValidatorType& ioIn,
@@ -120,6 +124,7 @@ public:
 	           SizeType nsectors,
 	           bool isObserveCode) :
 	    parameters_(parameters),
+	    model_(model),
 	    isObserveCode_(isObserveCode),
 	    isRestart_(parameters_.options.isSet("restart")),
 	    systemStack_(parameters_.options.isSet("shrinkStacksOnDisk"),
@@ -344,6 +349,17 @@ public:
 
 	const VectorVectorRealType& energies() const { return energiesFromFile_; }
 
+	const ModelType& model() const { return model_; }
+
+	std::pair<BasisWithOperatorsType, BasisWithOperatorsType>
+	hookForMultiInSituLrs(SizeType ind) const
+	{
+		SizeType sind = systemStack_.size() - ind - 1;
+		SizeType eind = envStack_.size() - ind - 1;
+		return std::pair<BasisWithOperatorsType, BasisWithOperatorsType>(systemStack_[sind],
+		                                                                 envStack_[eind]);
+	}
+
 private:
 
 	// legacy reading (use as fallback only)
@@ -392,7 +408,7 @@ private:
 
 		bool allInSystem = (parameters_.options.isSet("geometryallinsystem"));
 
-		PsimagLite::Vector<FiniteLoop>::Type vfl;
+		VectorFiniteLoopType vfl;
 		int lastSite = (allInSystem) ? totalSites-2 : totalSites/2-1; // must be signed
 		int prevDeltaSign = 1;
 		bool checkPoint = false;
@@ -406,7 +422,7 @@ private:
 
 		if (totalSites & 1) lastSite++;
 
-		ParametersType::readFiniteLoops(ioIn,vfl);
+		ParametersType::readFiniteLoops(ioIn, vfl, parameters_.truncationControl);
 
 		if (!parameters_.autoRestart)
 			checkFiniteLoops(vfl,totalSites,lastSite,prevDeltaSign,checkPoint);
@@ -414,7 +430,7 @@ private:
 		checkMvalues(vfl, hilbertOneSite);
 	}
 
-	void checkFiniteLoops(const PsimagLite::Vector<FiniteLoop>::Type& finiteLoop,
+	void checkFiniteLoops(const VectorFiniteLoopType& finiteLoop,
 	                      SizeType totalSites,
 	                      SizeType lastSite,
 	                      int prevDeltaSign,
@@ -428,7 +444,7 @@ private:
 
 		SizeType sopt = 0; // have we started saving yet?
 		for (SizeType i=0;i<finiteLoop.size();i++)  {
-			SizeType thisSaveOption = (finiteLoop[i].wantsSave());
+			SizeType thisSaveOption = (finiteLoop[i].wants("save"));
 			if (sopt == 1 && !(thisSaveOption&1)) {
 				s = "Error for finite loop number " + ttos(i) + "\n";
 				s += "Once you say 1 on a finite loop, then all";
@@ -468,7 +484,7 @@ private:
 		}
 	}
 
-	void checkMvalues(const PsimagLite::Vector<FiniteLoop>::Type& finiteLoop,
+	void checkMvalues(const VectorFiniteLoopType& finiteLoop,
 	                  SizeType hilbertOneSite) const
 	{
 		for (SizeType i = 0;i < finiteLoop.size(); ++i)  {
@@ -542,6 +558,7 @@ private:
 	}
 
 	const ParametersType& parameters_;
+	const ModelType& model_;
 	bool isObserveCode_;
 	bool isRestart_;
 	DiskOrMemoryStackType systemStack_;
