@@ -87,7 +87,7 @@ DISCLOSED WOULD NOT INFRINGE PRIVATELY OWNED RIGHTS.
 #include "DmrgSerializer.h"
 #include "Recovery.h"
 #include "Truncation.h"
-#include "ObservablesInSitu.h"
+#include "ObservablesOnePointInSitu.h"
 #include "TargetSelector.h"
 #include "PsiBase64.h"
 #include "PrinterInDetail.h"
@@ -108,7 +108,8 @@ public:
 	typedef VectorWithOffsetType_ VectorWithOffsetType;
 	typedef typename ModelType::OperatorsType OperatorsType;
 	typedef typename OperatorsType::OperatorType OperatorType;
-	typedef ObservablesInSitu<typename TargetingType::TargetVectorType> ObservablesInSituType;
+	typedef ObservablesOnePointInSitu<typename TargetingType::TargetVectorType>
+	ObservablesOnePointInSituType;
 	typedef PsimagLite::Vector<PsimagLite::String>::Type VectorStringType;
 	typedef typename  OperatorsType::SparseMatrixType SparseMatrixType;
 	typedef typename ModelType::MyBasis MyBasis;
@@ -142,6 +143,7 @@ public:
 	typedef typename TargetingType::VectorVectorVectorWithOffsetType
 	VectorVectorVectorWithOffsetType;
 	typedef OneSiteTruncation<ModelType, VectorWithOffsetType> OneSiteTruncationType;
+	using FiniteLoopType = FiniteLoop<RealType>;
 
 	DmrgSolver(ModelType& model, InputValidatorType& ioIn)
 	    : model_(model),
@@ -226,7 +228,7 @@ public:
 		for (SizeType i=0;i<Y.size();i++) sitesIndices_.push_back(Y[Y.size()-i-1]);
 
 		TargetSelector<TargetingType> targetSelector(lrs_,
-		                                             model_,
+		                                             checkpoint_,
 		                                             wft_,
 		                                             quantumSector_,
 		                                             ioIn_);
@@ -368,7 +370,11 @@ obtain ordered
 			                 X[step],
 			                 ystep);
 
-			truncate_.changeBasisInfinite(pS, pE, psi, parameters_.keptStatesInfinite);
+			truncate_.changeBasisInfinite(pS,
+			                              pE,
+			                              psi,
+			                              parameters_.keptStatesInfinite,
+			                              parameters_.truncationControl);
 
 			if (needsRightPush) {
 				if (!twoSiteDmrg) checkpoint_.push(pS,pE);
@@ -446,7 +452,7 @@ obtain ordered
 				}
 			}
 
-			const SizeType wantsOneSiteTrunc = (parameters_.finiteLoop[i].wantsOneSiteTruncation())
+			const SizeType wantsOneSiteTrunc = (parameters_.finiteLoop[i].wants("OneSiteTruncation"))
 			        ? 1 : 0;
 			model_.announce("finite loop;" + ttos(wantsOneSiteTrunc));
 
@@ -454,7 +460,7 @@ obtain ordered
 
 			if (psi.end()) break;
 
-			if (recovery.byLoop(i))
+			if (recovery.byLoop(i, psi.time(), parameters_.finiteLoop[i].stepLength()))
 				recovery.write(psi,
 				               i + 1,
 				               stepCurrent_,
@@ -583,7 +589,7 @@ obtain ordered
 				oneSiteTruncation_.update(oneSiteTruncSize, *(target.psiConst()[0][0]), direction);
 			}
 
-			changeTruncateAndSerialize(pS,pE,target,keptStates,direction,loopIndex);
+			changeTruncateAndSerialize(pS, pE, target, direction, loopIndex);
 
 			if (finalStep(stepLength, stepFinal)) break;
 
@@ -604,16 +610,21 @@ obtain ordered
 	void changeTruncateAndSerialize(MyBasisWithOperators& pS,
 	                                MyBasisWithOperators& pE,
 	                                const TargetingType& target,
-	                                SizeType keptStates,
 	                                ProgramGlobals::DirectionEnum direction,
 	                                SizeType loopIndex)
 	{
 		const bool twoSiteDmrg = parameters_.options.isSet("twositedmrg");
 		FermionSignType fsS(pS.signs());
-
 		FermionSignType fsE(pE.signs());
+		assert(loopIndex < parameters_.finiteLoop.size());
+		const FiniteLoopType& finiteLoop = parameters_.finiteLoop[loopIndex];
 
-		truncate_.changeBasisFinite(pS, pE, target, keptStates, direction);
+		truncate_.changeBasisFinite(pS,
+		                            pE,
+		                            target,
+		                            finiteLoop.keptStates(),
+		                            finiteLoop.truncationControl(),
+		                            direction);
 
 		if (direction == ProgramGlobals::DirectionEnum::EXPAND_SYSTEM)
 			checkpoint_.push((twoSiteDmrg) ? lrs_.left() : pS,
@@ -633,10 +644,9 @@ obtain ordered
 	{
 		if (!saveData_) return;
 
-		const FiniteLoop& finiteLoop = parameters_.finiteLoop[loopIndex];
-
-		if (!finiteLoop.wantsSave() && !finiteLoop.wantsMultiSitePush()) return;
+		const FiniteLoopType& finiteLoop = parameters_.finiteLoop[loopIndex];
 
+		if (!finiteLoop.wants("save") && !finiteLoop.wants("multisitepush")) return;
 
 		const BlockDiagonalMatrixType& transform = truncate_.transform(direction);
 		// FIXME: Serializer will for now save only one psi target
@@ -653,12 +663,12 @@ obtain ordered
 		                                                psi,
 		                                                transform,
 		                                                direction);
-		if (finiteLoop.wantsMultiSitePush()) {
+		if (finiteLoop.wants("multisitepush")) {
 			target.multiSitePush(ds);
 			return;
 		}
 
-		typename BasisWithOperatorsType::SaveEnum saveOption2 = (finiteLoop.wantsOnlySlowWft())
+		typename BasisWithOperatorsType::SaveEnum saveOption2 = (finiteLoop.wants("onlyslowwft"))
 		        ? BasisWithOperatorsType::SaveEnum::ALL
 		        : BasisWithOperatorsType::SaveEnum::PARTIAL;
 		SizeType numberOfSites = model_.superGeometry().numberOfSites();
@@ -739,7 +749,7 @@ obtain ordered
 	VectorBlockType sitesIndices_;
 	DiagonalizationType diagonalization_;
 	TruncationType truncate_;
-	ObservablesInSituType inSitu_;
+	ObservablesOnePointInSituType inSitu_;
 	bool saveData_;
 	OneSiteTruncationType oneSiteTruncation_;
 	static bool firstCall_;
