@@ -185,7 +185,8 @@ public:
 		knownLabels_.push_back("TruncationTolerance");
 		knownLabels_.push_back("PotentialT");
 		knownLabels_.push_back("omega");
-		knownLabels_.push_back("MagneticField");
+		knownLabels_.push_back("MagneticFieldX");
+		knownLabels_.push_back("MagneticFieldZ");
 		knownLabels_.push_back("SpinOrbit");
 		knownLabels_.push_back("DegeneracyMax");
 		knownLabels_.push_back("JzSymmetry");
@@ -213,6 +214,7 @@ public:
 		knownLabels_.push_back("CVnForFraction");
 		knownLabels_.push_back("AnisotropyD");
 		knownLabels_.push_back("FindSymmetrySector");
+		knownLabels_.push_back("AddOnSiteHamiltonian");
 
 		for (SizeType i = 0; i < 10; ++i)
 			knownLabels_.push_back("Term" + ttos(i));
@@ -278,6 +280,9 @@ public:
 		str += "integer CVnForFraction;\n";
 		str += "real AnisotropyD;\n";
 		str += "string FindSymmetrySector;\n";
+		str += "string AddOnSiteHamiltonian;\n";
+		str += "vector MagneticFieldX;\n";
+		str += "vector MagneticFieldZ;\n";
 
 		return str;
 	}
@@ -380,7 +385,7 @@ public:
 			\item [BatchedGemm] Only meaningful with MatrixVectorKron. Enables
 								batched gemm and might need plugin sc
 			\item [KrylovNoAbridge] TBW
-			\item [fixLegacyBugs] TBW
+			\item [keepLegacyBugs] TBW
 			\item [KronNoUseLowerPart] Don't Use lower part of Kron matrix but
  recompute it instead.
 			\item [shrinkStacksOnDisk] Store shrink stacks on disk instead of in memory
@@ -455,7 +460,7 @@ public:
 		registerOpts.push_back("wftAccelPatches");
 		registerOpts.push_back("BatchedGemm");
 		registerOpts.push_back("KrylovNoAbridge");
-		registerOpts.push_back("fixLegacyBugs");
+		registerOpts.push_back("keepLegacyBugs");
 		registerOpts.push_back("KronNoUseLowerPart");
 		registerOpts.push_back("shrinkStacksOnDisk");
 		registerOpts.push_back("OperatorsChangeAll");
