@@ -93,9 +93,36 @@ DISCLOSED WOULD NOT INFRINGE PRIVATELY OWNED RIGHTS.
 #include "GetBraOrKet.h"
 #include "RestartStruct.h"
 #include "SdhsReinterpret.h"
+#include "MultiPointInSitu.h"
+#include "Checkpoint.h"
 
 namespace Dmrg {
 
+template<typename LambdaType>
+class LambdaForTests {
+
+public:
+
+	void operator()(const PsimagLite::GetBraOrKet& bra,
+	                const PsimagLite::GetBraOrKet& ket,
+	                LambdaType* t) const
+	{
+		if (!t) return;
+		t->operator()(bra, ket);
+	}
+};
+
+template<>
+class LambdaForTests<int> {
+
+public:
+
+	void operator()(const PsimagLite::GetBraOrKet&,
+	                const PsimagLite::GetBraOrKet&,
+	                int*) const
+	{}
+};
+
 template<typename TargetHelperType,
          typename VectorWithOffsetType_,
          typename LanczosSolverType_>
@@ -137,6 +164,7 @@ public:
 	VectorVectorWithOffsetType;
 	typedef typename ApplyOperatorExpressionType::VectorRealType VectorRealType;
 	typedef typename ApplyOperatorExpressionType::PairType PairType;
+	typedef typename ApplyOperatorExpressionType::TimeVectorsBaseType TimeVectorsBaseType;
 	typedef typename ModelType::InputValidatorType InputValidatorType;
 	typedef Braket<ModelType> BraketType;
 	typedef FermionSign FermionSignType;
@@ -146,27 +174,30 @@ public:
 	typedef typename ApplyOperatorExpressionType::VectorVectorVectorWithOffsetType
 	VectorVectorVectorWithOffsetType;
 	typedef SdhsReinterpret<BraketType> SdhsReinterpretType;
+	typedef MultiPointInSitu<VectorWithOffsetType_, ModelType> MultiPointInSituType;
+	typedef Checkpoint<ModelType, WaveFunctionTransfType> CheckpointType;
 
 	enum class OpLabelCategory { DRESSED, BARE };
 
 	TargetingCommon(const LeftRightSuperType& lrs,
-	                const ModelType& model,
+	                const CheckpointType& checkPoint,
 	                const WaveFunctionTransfType& wft,
 	                SizeType indexNoAdvance)
 	    : cocoonType_(OpLabelCategory::DRESSED),
 	      progress_("TargetingCommon"),
-	      targetHelper_(lrs, model, wft),
+	      checkPoint_(checkPoint),
+	      targetHelper_(lrs, checkPoint.model(), wft),
 	      aoe_(targetHelper_, indexNoAdvance),
-	      inSitu_(model.superGeometry().numberOfSites())
+	      inSitu_(checkPoint.model().superGeometry().numberOfSites())
 	{
-		PsimagLite::split(meas_, model.params().insitu, ",");
+		PsimagLite::split(meas_, checkPoint.model().params().insitu, ",");
 		SizeType n = meas_.size();
 		for (SizeType i = 0; i < n; ++i) {
 			const bool isDressed = isOpLabelDressed(meas_[i]);
 
 			// check this early that what's passed makes sense
 			if (isDressed)
-				BraketType(model, meas_[i]);
+				BraketType(checkPoint.model(), meas_[i]);
 
 			OpLabelCategory cocoonExpected = (isDressed) ? OpLabelCategory::DRESSED
 			                                             : OpLabelCategory::BARE;
@@ -220,11 +251,10 @@ public:
 	                PsimagLite::String name) const
 	{
 		SizeType site = block[0];
-		TimeSerializerType ts(aoe_.currentTimeStep(),
-		                      aoe_.time(),
+		TimeSerializerType ts(currentTimeStep(),
+		                      time(),
 		                      site,
-		                      aoe_.targetVectors(),
-		                      aoe_.stages(),
+		                      aoe_,
 		                      name);
 		ts.write(io, prefix);
 	}
@@ -267,7 +297,7 @@ public:
 			aoe_.setStage(i, stages[i]);
 
 		SizeType rtvs = ts->numberOfVectors();        // read tvs
-		SizeType dtvs = aoe_.targetVectors().size();  // destination tvs
+		SizeType dtvs = aoe_.tvs();  // destination tvs
 
 		int tvForPsi = checkpoint.sourceTvForPsi();
 		if (tvForPsi >= 0) {
@@ -281,6 +311,7 @@ public:
 			aoe_.setOnlyOnePsi(ts->vector(tvForPsiUnsigned));
 		}
 
+		ApplyOperatorExpressionType& aoeNonConst = const_cast<ApplyOperatorExpressionType&>(aoe_);
 		for (SizeType i = 0; i < dtvs; ++i) {
 			const int j = checkpoint.mappingTvs(i);
 			if (j < 0) continue;
@@ -290,16 +321,18 @@ public:
 				    ttos(j) + " >= " + ttos(rtvs) + "\n");
 			}
 
-			aoe_.targetVectors(i) = ts->vector(j);
+			aoeNonConst.targetVectorsNonConst(i) = ts->vector(j);
 		}
 
 		// FIXME TODO check that the NGST name changes instead
-		bool sameNgst = isThisNgstSameAsPrevious(name, ts->name(), dtvs, rtvs);
-		SizeType cTimeStep = (sameNgst) ? ts->currentTimeStep() : 0;
-		aoe_.setCurrentTimeStep(cTimeStep);
+		if (name == "TimeStep") {
+			bool sameNgst = isThisNgstSameAsPrevious(name, ts->name(), dtvs, rtvs);
+			SizeType cTimeStep = (sameNgst) ? ts->currentTimeStep() : 0;
+			setCurrentTimeStep(cTimeStep);
 
-		RealType timeReal = (sameNgst) ? ts->time() : 0;
-		aoe_.setCurrentTime(timeReal);
+			RealType timeReal = (sameNgst) ? ts->time() : 0;
+			setCurrentTime(timeReal);
+		}
 
 		delete ts;
 		ts = 0;
@@ -309,13 +342,20 @@ public:
 
 	const ApplyOperatorExpressionType& aoe() const { return aoe_; }
 
-	ApplyOperatorExpressionType& aoe() { return aoe_; }
+	ApplyOperatorExpressionType& aoeNonConst() { return aoe_; }
+
+	RealType time() const
+	{
+		return aoe_.timeVectors().time();
+	}
 
 	// START Cocoons
 
+	template<typename SomeLambdaType = int>
 	void cocoon(const BlockType& block,
 	            ProgramGlobals::DirectionEnum direction,
-	            bool doBorderIfBorder) const
+	            bool doBorderIfBorder,
+	            SomeLambdaType* someLambda = nullptr) const
 	{
 		if (aoe_.noStageIs(StageEnumType::DISABLED))
 			std::cout<<"ALL OPERATORS HAVE BEEN APPLIED\n";
@@ -339,11 +379,22 @@ public:
 
 		const SizeType expectedSize = targetHelper_.model().hilbertSize(site);
 
+		LambdaForTests<SomeLambdaType> lambdaForTests;
+		MultiPointInSituType multiPointInSitu(aoe_.model(),
+		                                      checkPoint_,
+		                                      targetHelper_.wft(),
+		                                      direction);
+
 		for (SizeType i = 0; i < n; ++i) {
 			PsimagLite::String opLabel = meas_[i];
 
 			BraketType braket(targetHelper_.model(), opLabel);
 
+			if (braket.points() != 1) {
+				multiPointInSitu(braket, site);
+				continue;
+			}
+
 			SdhsReinterpretType sdhs(braket, {site});
 
 			if (sdhs.forbidden()) continue;
@@ -360,6 +411,7 @@ public:
 
 			if (v1->size() == 0 || v2->size() == 0) continue;
 
+			lambdaForTests(braket.bra(), braket.ket(), someLambda);
 			test(*v1, *v2, direction, opLabel, site, nup, border);
 			// don't repeat for border because this is called twice if needed
 		}
@@ -439,7 +491,7 @@ public:
 
 	RealType normSquared(SizeType i) const
 	{
-		const VectorWithOffsetType& v = aoe_.targetVectors()[i];
+		const VectorWithOffsetType& v = aoe_.targetVectors(i);
 		if (v.size() == 0) return 0;
 		// call to mult will conjugate one of the vector
 		return PsimagLite::real(v*v);
@@ -447,7 +499,7 @@ public:
 
 	void normalizeTimeVectors(SizeType start = 0, SizeType end = 0)
 	{
-		SizeType total =  aoe_.targetVectors().size();
+		SizeType total =  aoe_.tvs();
 		if (end == 0) end = total;
 		for (SizeType i = start; i < end; ++i) {
 			RealType factor = normSquared(i);
@@ -477,7 +529,8 @@ public:
 	                       const BlockType& block1)
 	{
 		const VectorWithOffsetType& psi = aoe_.ensureOnlyOnePsi("computeCorrection");
-		VectorWithOffsetType& v = aoe_.targetVectors(0);
+		ApplyOperatorExpressionType& aoeNonConst = const_cast<ApplyOperatorExpressionType&>(aoe_);
+		VectorWithOffsetType& v = aoeNonConst.targetVectorsNonConst(0);
 
 		// operators in the one-site basis:
 		typename PsimagLite::Vector<OperatorType>::Type creationMatrix;
@@ -650,16 +703,34 @@ public:
 			msg2<<normSquared(i)<<" ";
 		progress_.printline(msgg2, std::cout);
 
-		if (aoe_.currentTimeStep() == 0) return;
+		if (currentTimeStep() == 0) return;
 
 		PsimagLite::OstringStream msgg3(std::cout.precision());
 		PsimagLite::OstringStream::OstringStreamType& msg3 = msgg3();
-		msg3<<"CurrentTime="<<aoe_.time()<<" CurrentTimeStep="<<aoe_.currentTimeStep();
+		msg3<<"CurrentTime="<<time();
+		msg3<<" CurrentTimeStep="<<currentTimeStep();
 		progress_.printline(msgg3, std::cout);
 	}
 
 private:
 
+	SizeType currentTimeStep() const
+	{
+		return aoe_.timeVectors().currentTimeStep();
+	}
+
+	void setCurrentTimeStep(SizeType ts)
+	{
+		TimeVectorsBaseType* ptr = const_cast<TimeVectorsBaseType*>(&aoe_.timeVectors());
+		ptr->setCurrentTimeStep(ts);
+	}
+
+	void setCurrentTime(RealType t)
+	{
+		TimeVectorsBaseType* ptr = const_cast<TimeVectorsBaseType*>(&aoe_.timeVectors());
+		ptr->setCurrentTime(t);
+	}
+
 	bool isThisNgstSameAsPrevious(PsimagLite::String nameThis,
 	                              PsimagLite::String namePrev,
 	                              SizeType tvsThis,
@@ -741,7 +812,8 @@ private:
 	{
 		if (getBraOrKet.isPvector()) {
 			const SizeType pIndex = getBraOrKet.pIndex();
-			return &(aoe_.targetVectors(pIndex));
+			return (pIndex >= aoe_.tvs()) ? nullptr
+			                              : &(aoe_.targetVectors(pIndex));
 		}
 
 		const SizeType sectorIndex = getBraOrKet.sectorIndex();
@@ -769,7 +841,7 @@ private:
 	          BorderEnumType border) const
 	{
 		ComplexOrRealType sum = test_(src1,src2,systemOrEnviron,site,A,border);
-		std::cout<<site<<" "<<sum<<" "<<aoe_.time();
+		std::cout<<site<<" "<<sum<<" "<<time();
 		std::cout<<" "<<label<<" "<<(src1*src2)<<"\n";
 	}
 
@@ -795,6 +867,7 @@ private:
 	OpLabelCategory cocoonType_;
 	VectorStringType meas_;
 	PsimagLite::ProgressIndicator progress_;
+	const CheckpointType& checkPoint_;
 	TargetHelperType targetHelper_;
 	ApplyOperatorExpressionType aoe_;
 	mutable VectorType inSitu_;
