@@ -84,6 +84,7 @@ DISCLOSED WOULD NOT INFRINGE PRIVATELY OWNED RIGHTS.
 #include "Pvector.h"
 #include "SpecForTargetingExpression.h"
 #include "CanonicalExpression.h"
+#include "GroupOfOneTimeEvolutions.h"
 
 namespace Dmrg {
 
@@ -94,6 +95,7 @@ class TargetingExpression : public TargetingBase<LanczosSolverType_,VectorWithOf
 	typedef typename BaseType::TargetingCommonType TargetingCommonType;
 	typedef typename BaseType::WaveFunctionTransfType WaveFunctionTransfType;
 	typedef typename BaseType::ModelType ModelType;
+	typedef typename BaseType::CheckpointType CheckpointType;
 	typedef typename ModelType::RealType RealType;
 	typedef typename ModelType::InputValidatorType InputValidatorType;
 	typedef typename ModelType::ModelHelperType ModelHelperType;
@@ -104,10 +106,10 @@ class TargetingExpression : public TargetingBase<LanczosSolverType_,VectorWithOf
 	typedef typename BasisType::QnType QnType;
 	typedef typename TargetingCommonType::StageEnumType StageEnumType;
 	typedef Pvector<typename VectorWithOffsetType_::value_type> PvectorType;
-	typedef typename PsimagLite::Vector<PvectorType*>::Type VectorPvectorType;
 	typedef SpecForTargetingExpression<BaseType> SpecForTargetingExpressionType;
 	typedef typename SpecForTargetingExpressionType::AlgebraType AlgebraType;
-	typedef PsimagLite::CanonicalExpression<SpecForTargetingExpressionType>
+	typedef typename SpecForTargetingExpressionType::AssignAndDestroy AssignAndDestroyType;
+	typedef PsimagLite::CanonicalExpression<SpecForTargetingExpressionType, AssignAndDestroyType>
 	CanonicalExpressionType;
 	typedef AuxForTargetingExpression<BaseType> AuxForTargetingExpressionType;
 	typedef typename TargetingCommonType::VectorRealType VectorRealType;
@@ -116,89 +118,122 @@ class TargetingExpression : public TargetingBase<LanczosSolverType_,VectorWithOf
 	VectorVectorWithOffsetType;
 	typedef PsimagLite::Vector<bool>::Type VectorBoolType;
 	typedef typename AlgebraType::VectorSizeType VectorSizeType;
+	typedef typename BaseType::ApplyOperatorExpressionType ApplyOperatorExpressionType;
+	typedef GroupOfOneTimeEvolutions<Pvectors<BaseType> > GroupOfOneTimeEvolutionsType;
 
 public:
 
 	TargetingExpression(const LeftRightSuperType& lrs,
-	                    const ModelType& model,
+	                    const CheckpointType& checkPoint,
 	                    const WaveFunctionTransfType& wft,
 	                    const QnType&,
 	                    InputValidatorType& io)
-	    : BaseType(lrs,model,wft,0),
+	    : BaseType(lrs, checkPoint, wft, 0),
 	      progress_("TargetingExpression"),
 	      gsWeight_(0.3),
-	      origPvectors_(0)
+	      gsWeightActual_(gsWeight_),
+	      pvectors_(io, this->common().aoe(), lrs)
 	{
 		io.readline(gsWeight_, "GsWeight=");
-		pvectorsFromInput(io);
-	}
-
-	~TargetingExpression()
-	{
-		for (SizeType i = 0; i < pVectors_.size(); ++i) {
-			delete pVectors_[i];
-			pVectors_[i] = 0;
-		}
 	}
 
 	SizeType sites() const { return 0; }
 
-	SizeType targets() const { return pVectors_.size(); }
+	SizeType targets() const { return pvectors_.targets(); }
 
 	RealType normSquared(SizeType i) const
 	{
-		return PsimagLite::real(this->common().aoe().targetVectors()[i]*
-		                        this->common().aoe().targetVectors()[i]);
+		return PsimagLite::real(this->tv(i)*
+		                        this->tv(i));
 	}
 
 	RealType weight(SizeType i) const
 	{
 		assert(this->common().aoe().noStageIs(StageEnumType::DISABLED));
-		VectorRealType weights;
-		RealType gsWeight = 0;
-		computeAllWeights(gsWeight, weights);
-		assert(i < weights.size());
-		return weights[i];
+		assert(i < weights_.size());
+		return weights_[i];
 	}
 
 	RealType gsWeight() const
 	{
-		VectorRealType weights;
-		RealType gsWeight = 0;
-		computeAllWeights(gsWeight, weights);
-		return gsWeight;
+		return gsWeightActual_;
 	}
 
-	void evolve(const VectorRealType&,
+	void evolve(const VectorRealType& energies,
 	            ProgramGlobals::DirectionEnum direction,
 	            const BlockType& block1,
 	            const BlockType&,
-	            SizeType)
+	            SizeType loopNumber)
 	{
 		if (direction == ProgramGlobals::DirectionEnum::INFINITE) return;
 
 		this->common().setAllStagesTo(StageEnumType::WFT_NOADVANCE);
 		assert(block1.size() == 1);
 		const SizeType site = block1[0];
-		const SizeType total = this->common().aoe().targetVectors().size();
-		assert(total <= pVectors_.size());
-		this->common().aoe().wftSome(site, 0, total);
+		const SizeType numberOfSites = this->common().aoe().model().superGeometry().numberOfSites();
+		const SizeType total = this->common().aoe().tvs();
+		assert(total <= pvectors_.targets());
+		if (site != 0 && site + 1 != numberOfSites)
+			this->common().aoeNonConst().wftSome(site, 0, total);
 
-		computePvectors(direction); // may alter the number of tvs
+		assert(energies.size() > 0);
+		computePvectors(direction, energies[0], site); // may alter the number of tvs
 
-		VectorRealType weight;
-		RealType gsWeight = 0;
-		computeAllWeights(gsWeight, weight);
-		this->common().printNormsAndWeights(gsWeight, weight);
+		computeAllWeights();
+		this->common().printNormsAndWeights(gsWeightActual_, weights_);
 
 		const bool doBorderIfBorder = true;
-		this->common().cocoon(block1, direction, doBorderIfBorder); // in-situ
+		auto testLambda = [this](const PsimagLite::GetBraOrKet& bra,
+		        const PsimagLite::GetBraOrKet& ket)
+		{
+			if (!hasTimeEvolution(bra) || !hasTimeEvolution(ket)) return;
+
+			RealType braTime = getTimeForKet(bra);
+			RealType ketTime = getTimeForKet(ket);
+			const RealType globalTime = this->common().time();
+
+			if (braTime != ketTime)
+				err("BraTime= " + ttos(braTime) + " but ketTime= " + ttos(ketTime) + "\n");
+
+			if (braTime != globalTime)
+				err("Bra and kets have some time= " + ttos(braTime) +
+				    " but globalTime= " + ttos(globalTime) + "\n");
+		};
+
+		if (loopNumber >=this->model().params().finiteLoop.size()-1) {
+			if (direction == ProgramGlobals::DirectionEnum::EXPAND_SYSTEM) {
+				if (site >= numberOfSites - 1)
+					return;
+			} else {
+				if (site < 1)
+					return;
+			}
+		}
+
+		this->common().cocoon(block1, direction, doBorderIfBorder, &testLambda); // in-situ
+
+		// border trigger below
+
+		// avoid self-triggers:
+		if (site == 0 || site == numberOfSites - 1) return;
+
+		if (site > 1 && site < numberOfSites - 2) return;
+
+		if (direction == ProgramGlobals::DirectionEnum::EXPAND_SYSTEM) {
+			if (site == 1) return;
+		} else {
+			if (site == numberOfSites - 2) return;
+		}
+
+		SizeType x = (site == 1) ? 0 : numberOfSites - 1;
+		BlockType block(1, x);
+		evolve(energies, direction, block, block, loopNumber);
 	}
 
 	void read(typename TargetingCommonType::IoInputType& io,
 	          PsimagLite::String prefix)
 	{
-		this->common().read(io, prefix);
+		this->common().readGSandNGSTs(io, prefix, "Expression");
 	}
 
 	void write(const typename PsimagLite::Vector<SizeType>::Type& block,
@@ -209,36 +244,30 @@ public:
 		this->common().writeNGSTs(io, prefix, block, "Expression");
 	}
 
-private:
-
-	void pvectorsFromInput(InputValidatorType& io)
+	bool hasTimeEvolution(const PsimagLite::GetBraOrKet& ket) const
 	{
-		SizeType total = 0;
-		io.readline(total, "Pvectors=");
-		pVectors_.resize(total);
-		PsimagLite::String tmp;
-		RealType sum = 0.0;
-		for (SizeType i = 0; i < total; ++i) {
-			io.readline(tmp, "P" + ttos(i) + "=");
-			pVectors_[i] = new PvectorType(tmp);
-			sum += pVectors_[i]->weight();
-		}
-
-		if (sum == 0.0) return;
-
-		RealType factor = (1.0 - gsWeight_)/sum;
-		for (SizeType i = 0; i < total; ++i)
-			pVectors_[i]->multiplyWeight(factor);
-
-		origPvectors_ = pVectors_.size();
+		return (ket.isPvector()) ? pvectors_.hasTimeEvolution(ket.pIndex()) : false;
 	}
 
-	void computePvectors(ProgramGlobals::DirectionEnum dir)
+	RealType getTimeForKet(const PsimagLite::GetBraOrKet& ket) const
+	{
+		if (ket.isPvector())
+			return timeEvolve_.getTimeForKet(ket.pIndex());
+
+		if (ket.isRvector())
+			throw PsimagLite::RuntimeError("R vectors cannot be tested\n");
+
+		return 0;
+	}
+
+private:
+
+	void computePvectors(ProgramGlobals::DirectionEnum dir, RealType Eg, SizeType site)
 	{
 		if (allOrigPvectorsDone()) {
-			const SizeType tvs = this->common().aoe().targetVectors().size();
-			if (tvs == origPvectors_) return;
-			if (tvs < origPvectors_)
+			const SizeType tvs = this->common().aoe().tvs();
+			if (tvs == pvectors_.origPvectors()) return;
+			if (tvs < pvectors_.origPvectors())
 				err("TVS could not have decreased ?!\n");
 
 			PsimagLite::OstringStream msgg(std::cout.precision());
@@ -249,45 +278,48 @@ private:
 			std::cerr<<"All user-provided P vectors finished\n";
 			std::cerr<<PsimagLite::AnsiColor::reset;
 
-			this->common().aoe().targetVectorsResize(origPvectors_);
+			this->common().aoeNonConst().targetVectorsResize(pvectors_.origPvectors());
 
 			return;
 		}
 
 		PsimagLite::OstringStream msgg(std::cout.precision());
 		PsimagLite::OstringStream::OstringStreamType& msg = msgg();
-		msg<<"P0="<<pVectors_[0]->lastName();
+		msg<<"P0="<<pvectors_(0).lastName();
 		progress_.printline(msgg, std::cout);
 
 		CanonicalExpressionType canonicalExpression(opSpec_);
-		SizeType total = pVectors_.size();
+		SizeType total = pvectors_.targets();
 
-		AuxForTargetingExpressionType aux(this->common().aoe(),
-		                                  this->model(),
-		                                  this->lrs(),
-		                                  dir);
-		const AlgebraType opEmpty(aux);
+		AuxForTargetingExpressionType aux(pvectors_, timeEvolve_, dir, Eg, site);
+		AlgebraType opEmpty(aux);
 		bool needsTrimming = false;
 		PsimagLite::String allpvectors;
 		for (SizeType i = 0; i < total; ++i) {
 
-			if (pVectors_[i]->lastName() == "DONE") continue;
+			if (pvectors_(i).isDone()) continue;
 
+			aux.setPindexOutput(i);
 			AlgebraType tmp(aux);
 
-			canonicalExpression(tmp, pVectors_[i]->lastName(), opEmpty, aux);
+			canonicalExpression(tmp, pvectors_(i).lastName(), opEmpty, aux);
 			tmp.finalize();
 
 			PsimagLite::String thispBefore = tmp.toString();
 			finalize(aux.tempVectors(), aux.tempNames(), i, thispBefore);
-			PsimagLite::String thispAfter = pVectors_[i]->lastName();
+			PsimagLite::String thispAfter = pvectors_(i).lastName();
 
 			int x = tmp.pIndex();
 			if (x >= 0) {
-				if (static_cast<SizeType>(x) == i) err("Self assigment\n");
-				VectorWithOffsetType_& v0 = this->common().aoe().targetVectors(i);
-				v0 =  this->common().aoe().targetVectors(x);
-				pVectors_[i]->pushString("DONE");
+				if (static_cast<SizeType>(x) != i) {
+					VectorWithOffsetType_& v0 = this->tvNonConst(i);
+					v0 =  this->tv(x);
+				} else {
+					std::cerr<<"Ignoring self assignment P";
+					std::cerr<<i<<"=P"<<x<<"\n";
+				}
+
+				pvectors_.setAsDone(i);
 				continue;
 			}
 
@@ -297,18 +329,18 @@ private:
 			}
 
 			PsimagLite::String newpstring = simplifyTerms(thispBefore);
-			if (newpstring != pVectors_[i]->lastName()) {
+			if (newpstring != pvectors_(i).lastName()) {
 				needsTrimming = true;
 				PsimagLite::String compr = compressExpression(newpstring);
 				//checkNoUncompressedExists(compr);
-				pVectors_[i]->pushString(compr);
+				pvectors_.pushString(i, compr);
 				newpstring += compr;
 			}
 
 			allpvectors += newpstring;
 		}
 
-		if (needsTrimming) trimPvectors(allpvectors);
+		if (needsTrimming) pvectors_.trimPvectors(allpvectors);
 	}
 
 	void finalize(const VectorVectorWithOffsetType& tempVectors,
@@ -324,10 +356,10 @@ private:
 		VectorBoolType removed_(ntemps);
 		VectorSizeType tempToP(ntemps, 10000);
 		for (SizeType i = 0; i < ntemps; ++i) {
-			int x = findInOrigNames(tempNames[i]);
+			int x = pvectors_.findInOrigNames(tempNames[i]);
 			if (x < 0) continue;
-			this->common().aoe().targetVectors(x) = tempVectors[i];
-			pVectors_[x]->pushString("DONE");
+			this->tvNonConst(x) = tempVectors[i];
+			pvectors_.setAsDone(x);
 			removed_[i] = true;
 			tempToP[i] = x;
 		}
@@ -335,53 +367,31 @@ private:
 		// these surviving tempNames_ need storage, add them
 		for (SizeType i = 0; i < ntemps; ++i) {
 			if (removed_[i]) continue;
-			int x = findInAnyNames(tempNames[i]);
+			int x = pvectors_.findInAnyNames(tempNames[i]);
 			if (x >= 0) continue;
-			const SizeType ind = this->common().aoe().createPvector(tempVectors[i]);
-			tempToP[i] = ind;
-			const PsimagLite::String ename = expandExpression(tempNames[i], tempToP);
-			PvectorType* pnew = new PvectorType(ename);
-			pnew->pushString("DONE");
-			pVectors_.push_back(pnew);
+			auto lambda = [this, i, &tempToP, &tempNames](SizeType ind) {
+				tempToP[i] = ind;
+				return this->expandExpression(tempNames[i], tempToP);
+			};
 
-			PsimagLite::OstringStream msgg(std::cout.precision());
-			PsimagLite::OstringStream::OstringStreamType& msg = msgg();
-			msg<<"P["<<ind<<"]="<<ename<<" created";
-			progress_.printline(msgg, std::cout);
+			pvectors_.createNew(tempVectors[i], lambda);
 		}
 
 		PsimagLite::String newpstring = compressExpression(tempExpr);
 		const PsimagLite::String selfName = "|P" + ttos(pVectorIndex) + ">";
-		if (newpstring == selfName) newpstring = "DONE";
-		pVectors_[pVectorIndex]->pushString(newpstring);
+		if (newpstring == selfName)
+			pvectors_.setAsDone(pVectorIndex);
+		else
+			pvectors_.pushString(pVectorIndex, newpstring);
 	}
 
 	bool allOrigPvectorsDone() const
 	{
-		for (SizeType i = 0; i < origPvectors_; ++i)
-			if (pVectors_[i]->lastName() != "DONE") return false;
+		for (SizeType i = 0; i < pvectors_.origPvectors(); ++i)
+			if (!pvectors_(i).isDone()) return false;
 		return true;
 	}
 
-	int findInOrigNames(PsimagLite::String str) const
-	{
-		return findInNames(str, origPvectors_);
-	}
-
-	int findInAnyNames(PsimagLite::String str) const
-	{
-		return findInNames(str, pVectors_.size());
-	}
-
-	int findInNames(PsimagLite::String str, SizeType end) const
-	{
-		assert(end <= pVectors_.size());
-		for (SizeType i = 0; i < end; ++i)
-			if (pVectors_[i]->hasAnyName(str)) return i;
-
-		return -1;
-	}
-
 	// replace "|!m" + something ==> "|P" + number
 	PsimagLite::String compressExpression(PsimagLite::String str) const
 	{
@@ -398,7 +408,7 @@ private:
 					if (str[j] == '>') break;
 				}
 
-				const SizeType ind = findPforThisExpression(buffer);
+				const SizeType ind = pvectors_.findPforThisExpression(buffer);
 				buffer = "|P" + ttos(ind) + ">";
 				i = j;
 				result += buffer;
@@ -411,17 +421,6 @@ private:
 		return result;
 	}
 
-	SizeType findPforThisExpression(PsimagLite::String str) const
-	{
-		const SizeType pvectors = pVectors_.size();
-		for (SizeType i = 0; i < pvectors; ++i) {
-			if (!pVectors_[i]->hasAnyName(str)) continue;
-			return i;
-		}
-
-		throw PsimagLite::RuntimeError("findPforThisExpression: P not found\n");
-	}
-
 	// replace "R" + i ==> "P" + tempToP[i]
 	PsimagLite::String expandExpression(PsimagLite::String str,
 	                                    const VectorSizeType& tempToP) const
@@ -494,7 +493,7 @@ private:
 					ind1 = tmp;
 				}
 
-				sumPvectors(ind0, ind1, p0PlusP1);
+				pvectors_.sumPvectors(ind0, ind1, p0PlusP1);
 
 				buffer = "|P" + ttos(ind0) + ">";
 				i = j;
@@ -519,105 +518,39 @@ private:
 		}
 	}
 
-	void sumPvectors(SizeType ind0, SizeType ind1, PsimagLite::String p0PlusP1)
+	void computeAllWeights()
 	{
-		assert(ind0 < ind1);
-		VectorWithOffsetType_& v0 = this->common().aoe().targetVectors(ind0);
-		const VectorWithOffsetType_& v1 =  this->common().aoe().targetVectors(ind1);
-		v0 += v1;
-		pVectors_[ind0]->sum(*(pVectors_[ind1]), p0PlusP1);
-	}
-
-	void trimPvectors(PsimagLite::String str)
-	{
-		const SizeType tvs = this->common().aoe().targetVectors().size();
-		VectorBoolType used(tvs, false);
-		assert(origPvectors_ <= tvs);
-		for (SizeType i = 0; i < origPvectors_; ++i)
-			used[i] = true;
-
-		findUsedPvectors(used, str);
-
-		for (SizeType i = 0; i < tvs; ++i) {
-			if (used[i]) continue;
-			this->common().aoe().destroyPvector(i);
-			PsimagLite::OstringStream msgg(std::cout.precision());
-			PsimagLite::OstringStream::OstringStreamType& msg = msgg();
-			msg<<"P["<<i<<"] destroyed";
-			progress_.printline(msgg, std::cout);
-		}
-
-		// this line is commented out because then the index of aoe.targetVectors
-		// will be different than the index of pVectors
-		// aoe.targetVectors is resized after all vectors have been computed
-		//this->common().aoe().trimVectors();
-		const SizeType tvsFinal = this->common().aoe().targetVectors().size();
-		if (tvs != tvsFinal) {
-			PsimagLite::OstringStream msgg(std::cout.precision());
-			PsimagLite::OstringStream::OstringStreamType& msg = msgg();
-			msg<<"Number of target vectors is "<<tvsFinal<<" now";
-			progress_.printline(msgg, std::cout);
-		}
-	}
-
-	void findUsedPvectors(VectorBoolType& used, PsimagLite::String str) const
-	{
-		const SizeType len = str.size();
-		for (SizeType i = 0; i < len; ++i) {
-
-			if (str.substr(i, 2) != "|P") continue;
-
-			PsimagLite::String buffer;
-			SizeType j = i + 2;
-			for (; j < len; ++j) {
-				if (str[j] >= 48 && str[j] <= 57) {
-					buffer += str[j];
-				} else if (str[j] == '>') {
-					break;
-				} else {
-					buffer = "";
-					break;
-				}
-			}
-
-			i = j;
-			if (buffer == "") continue;
-			const SizeType ind = PsimagLite::atoi(buffer);
-			used[ind] = true;
-		}
-	}
-
-	void computeAllWeights(RealType& gsWeight, VectorRealType& weight) const
-	{
-		const SizeType n = this->common().aoe().targetVectors().size();
-		assert(n <= pVectors_.size());
-		weight.resize(n);
-		std::fill(weight.begin(), weight.end(), 0);
+		const SizeType n = this->common().aoe().tvs();
+		assert(n <= pvectors_.targets());
+		weights_.resize(n);
+		std::fill(weights_.begin(), weights_.end(), 0);
 		RealType sum = 0;
 		for (SizeType i = 0; i < n; ++i) {
-			RealType norma = norm(this->common().aoe().targetVectors()[i]);
+			RealType norma = norm(this->tv(i));
 			if (norma < 1e-6) continue;
-			weight[i] = pVectors_[i]->weight()/norma;
-			sum += weight[i];
+			weights_[i] = pvectors_(i).weight()/norma;
+			sum += weights_[i];
 		}
 
-		gsWeight = 1 - sum;
+		gsWeightActual_ = 1 - sum;
 
-		if (gsWeight >= gsWeight_) return; // <--- EARLY EXIT HERE
+		if (gsWeightActual_ >= gsWeight_) return; // <--- EARLY EXIT HERE
 
 		assert(sum > 1e-6);
 		RealType factor = (1 - gsWeight_)/sum;
 		for (SizeType i = 0; i < n; ++i)
-			weight[i] *= factor;
+			weights_[i] *= factor;
 
-		gsWeight = gsWeight_;
+		gsWeightActual_ = gsWeight_;
 	}
 
 	PsimagLite::ProgressIndicator progress_;
 	RealType gsWeight_;
-	SizeType origPvectors_;
-	VectorPvectorType pVectors_;
+	RealType gsWeightActual_;
+	VectorRealType weights_;
 	SpecForTargetingExpressionType opSpec_;
+	Pvectors<BaseType> pvectors_;
+	GroupOfOneTimeEvolutionsType timeEvolve_;
 };     //class TargetingExpression
 } // namespace Dmrg
 /*@}*/
