@@ -113,6 +113,7 @@ public:
 	typedef typename BaseType::OptionsType OptionsType;
 	typedef typename BaseType::TargetingCommonType TargetingCommonType;
 	typedef typename BaseType::MatrixVectorType MatrixVectorType;
+	typedef typename BaseType::CheckpointType CheckpointType;
 	typedef typename MatrixVectorType::ModelType ModelType;
 	typedef typename ModelType::RealType RealType;
 	typedef typename ModelType::OperatorsType OperatorsType;
@@ -143,8 +144,6 @@ public:
 	TargetParamsType> MettsCollapseType;
 	typedef typename MettsCollapseType::PackIndicesType PackIndicesType;
 	typedef typename TargetingCommonType::TimeSerializerType TimeSerializerType;
-	typedef TimeVectorsBase<TargetParamsType,ModelType,WaveFunctionTransfType,
-	LanczosSolverType,VectorWithOffsetType> TimeVectorsBaseType;
 	typedef TimeVectorsKrylov<TargetParamsType,ModelType,WaveFunctionTransfType,
 	LanczosSolverType,VectorWithOffsetType> TimeVectorsKrylovType;
 	typedef TimeVectorsRungeKutta<TargetParamsType,ModelType,WaveFunctionTransfType,
@@ -157,22 +156,23 @@ public:
 	typedef typename PsimagLite::Vector<BlockDiagonalMatrixType*>::Type
 	VectorBlockDiagonalMatrixType;
 	typedef typename TargetingCommonType::StageEnumType StageEnumType;
+	typedef typename TargetingCommonType::ApplyOperatorExpressionType ApplyOperatorExpressionType;
+	typedef typename ApplyOperatorExpressionType::TimeVectorsBaseType TimeVectorsBaseType;
 
 	TargetingMetts(const LeftRightSuperType& lrs,
-	               const ModelType& model,
+	               const CheckpointType& checkPoint,
 	               const WaveFunctionTransfType& wft,
 	               const QnType& quantumSector,
 	               InputValidatorType& ioIn)
-	    : BaseType(lrs, model, wft, 0),
-	      model_(model),
+	    : BaseType(lrs, checkPoint, wft, 0),
+	      model_(checkPoint.model()),
 	      lrs_(lrs),
-	      mettsStruct_(ioIn, "TargetingMetts", model),
+	      mettsStruct_(ioIn, "TargetingMetts", checkPoint.model()),
 	      wft_(wft),
 	      quantumSector_(quantumSector),
 	      progress_("TargetingMetts"),
-	      betas_(mettsStruct_.timeSteps()),
-	      weight_(betas_.size() + 1),
-	      mettsStochastics_(model,mettsStruct_.rngSeed,mettsStruct_.pure),
+	      weight_(mettsStruct_.times().size() + 1),
+	      mettsStochastics_(checkPoint.model(), mettsStruct_.rngSeed, mettsStruct_.pure),
 	      mettsCollapse_(mettsStochastics_,lrs,mettsStruct_),
 	      prevDirection_(ProgramGlobals::DirectionEnum::INFINITE),
 	      systemPrev_(),
@@ -182,13 +182,13 @@ public:
 
 		if (!wft.isEnabled()) err(" TargetingMetts needs an enabled wft\n");
 
-		const RealType tau = mettsStruct_.tau()/(mettsStruct_.timeSteps()-1);
-		for (SizeType i = 0; i< betas_.size(); ++i)
-			betas_[i] = i*tau;
+		const RealType tau = mettsStruct_.tau()/(mettsStruct_.times().size()-1);
+		for (SizeType i = 0; i< mettsStruct_.times().size(); ++i)
+			mettsStruct_.times()[i] = i*tau;
 
 		setWeights();
 
-		this->common().aoe().initTimeVectors(mettsStruct_, betas_, ioIn);
+		this->common().aoeNonConst().initTimeVectors(mettsStruct_, ioIn);
 	}
 
 	~TargetingMetts()
@@ -202,7 +202,7 @@ public:
 
 	SizeType sites() const { return mettsStruct_.sites(); }
 
-	SizeType targets() const { return mettsStruct_.timeSteps() + 1; }
+	SizeType targets() const { return mettsStruct_.times().size() + 1; }
 
 	RealType weight(SizeType i) const
 	{
@@ -220,8 +220,8 @@ public:
 	{
 		if (this->common().aoe().allStages(StageEnumType::DISABLED))
 			return 1;
-		SizeType n = this->common().aoe().targetVectors().size();
-		if (this->common().aoe().targetVectors()[n-1].size()==0) n--;
+		SizeType n = this->common().aoe().tvs();
+		if (this->tv(n-1).size()==0) n--;
 		return n;
 	}
 
@@ -238,7 +238,7 @@ public:
 			utils::blockUnion(sites,block1,block2);
 		else sites = block1;
 
-		SizeType n1 = mettsStruct_.timeSteps();
+		SizeType n1 = mettsStruct_.times().size();
 
 		if (direction == ProgramGlobals::DirectionEnum::INFINITE) {
 			updateStochastics(block1,block2);
@@ -263,12 +263,12 @@ public:
 		calcTimeVectors(PairType(0,n1),Eg,direction,block1);
 
 		// Advance or wft  collapsed vector
-		if (this->common().aoe().targetVectors()[n1].size()>0)
+		if (this->tv(n1).size()>0)
 			evolve(n1,n1,n1-1,Eg,direction,sites,loopNumber);
 
-		for (SizeType i=0;i<this->common().aoe().targetVectors().size();i++)
-			assert(this->common().aoe().targetVectors()[i].size()==0 ||
-			       this->common().aoe().targetVectors()[i].size()==
+		for (SizeType i=0;i<this->common().aoe().tvs();i++)
+			assert(this->tv(i).size()==0 ||
+			       this->tv(i).size()==
 			       lrs_.super().permutationVector().size());
 
 		bool doBorderIfBorder = true;
@@ -294,8 +294,8 @@ public:
 		if (this->common().aoe().noStageIs(StageEnumType::COLLAPSE)) return;
 
 		// collapse
-		bool hasCollapsed = mettsCollapse_(this->common().aoe().targetVectors(n1),
-		                                   this->common().aoe().targetVectors()[n1-1],
+		bool hasCollapsed = mettsCollapse_(this->tvNonConst(n1),
+		                                   this->tv(n1 - 1),
 		        sites,
 		        direction);
 		if (hasCollapsed) {
@@ -317,11 +317,11 @@ public:
 	{
 		this->common().write(io, block, prefix);
 
-		VectorVectorWithOffsetType& tv = const_cast<VectorVectorWithOffsetType&>
-		        (this->common().aoe().targetVectors());
-		if (mettsStruct_.beta > this->common().aoe().time()) {
-			for (SizeType i = 0; i < this->common().aoe().targetVectors().size(); ++i)
-				tv[i].clear();
+		ApplyOperatorExpressionType& aoeNonConst = const_cast<ApplyOperatorExpressionType&>
+		        (this->common().aoe());
+		if (mettsStruct_.beta > this->common().aoe().timeVectors().time()) {
+			for (SizeType i = 0; i < this->common().aoe().tvs(); ++i)
+				aoeNonConst.destroyPvector(i);
 		}
 
 		this->common().writeNGSTs(io, prefix, block, "Metts");
@@ -360,20 +360,21 @@ private:
 	                     ProgramGlobals::DirectionEnum systemOrEnviron,
 	                     const VectorSizeType& block)
 	{
-		const VectorWithOffsetType& phi = this->common().aoe().targetVectors()[startEnd.first];
+		const VectorWithOffsetType& phi = this->tv(startEnd.first);
 		PsimagLite::OstringStream msgg(std::cout.precision());
 		PsimagLite::OstringStream::OstringStreamType& msg = msgg();
 		msg<<" vector number "<<startEnd.first<<" has norm ";
 		msg<<norm(phi);
 		progress_.printline(msgg, std::cout);
 		if (norm(phi)<1e-6)
-			setFromInfinite(this->common().aoe().targetVectors(startEnd.first),lrs_);
+			setFromInfinite(this->tvNonConst(startEnd.first),
+			                lrs_);
 		bool allOperatorsApplied = (this->common().aoe().noStageIs(StageEnumType::DISABLED));
 		VectorSizeType indices(startEnd.second - startEnd.first);
 		for (SizeType i = 0; i < indices.size(); ++i) indices[i] = i + startEnd.first;
 
 		const bool isLastCall = true;
-		this->common().aoe().calcTimeVectors(indices,
+		this->common().aoeNonConst().calcTimeVectors(indices,
 		                                     Eg,
 		                                     phi,
 		                                     systemOrEnviron,
@@ -400,16 +401,18 @@ private:
 			sitesCollapsed_.clear();
 			this->common().setAllStagesTo(StageEnumType::WFT_NOADVANCE);
 			timesWithoutAdvancement_ = 0;
-			this->common().aoe().setCurrentTimeStep(0);
+			TimeVectorsBaseType* ptr = const_cast<TimeVectorsBaseType*>(&(this->common().aoe().timeVectors()));
+			ptr->setCurrentTimeStep(0);
 			PsimagLite::OstringStream msgg(std::cout.precision());
 			PsimagLite::OstringStream::OstringStreamType& msg = msgg();
-			SizeType n1 = mettsStruct_.timeSteps();
-			RealType x = norm(this->common().aoe().targetVectors()[n1]);
+			SizeType n1 = mettsStruct_.times().size();
+			RealType x = norm(this->tv(n1));
 			msg<<"Changing direction, setting collapsed with norm="<<x;
 			progress_.printline(msgg, std::cout);
 			for (SizeType i=0;i<n1;i++)
-				this->common().aoe().targetVectors(i) = this->common().aoe().targetVectors()[n1];
-			this->common().aoe().timeHasAdvanced();
+				this->tvNonConst(i) =
+			        this->tv(n1);
+			this->common().aoeNonConst().timeHasAdvanced();
 			printAdvancement(timesWithoutAdvancement_);
 			return;
 		}
@@ -421,28 +424,29 @@ private:
 		}
 
 		if (this->common().aoe().noStageIs(StageEnumType::COLLAPSE) &&
-		        this->common().aoe().time() < mettsStruct_.beta) {
+		        this->common().aoe().timeVectors().time() < mettsStruct_.beta) {
 			this->common().setAllStagesTo(StageEnumType::WFT_ADVANCE);
-			const SizeType tmp = this->common().aoe().currentTimeStep() + 1;
-			this->common().aoe().setCurrentTimeStep(tmp);
+			const SizeType tmp = this->common().aoe().timeVectors().currentTimeStep() + 1;
+			TimeVectorsBaseType* ptr = const_cast<TimeVectorsBaseType*>(&(this->common().aoe().timeVectors()));
+			ptr->setCurrentTimeStep(tmp);
 			timesWithoutAdvancement_ = 0;
 			printAdvancement(timesWithoutAdvancement_);
 			return;
 		}
 
 		if (this->common().aoe().noStageIs(StageEnumType::COLLAPSE) &&
-		        this->common().aoe().time() >= mettsStruct_.beta &&
+		        this->common().aoe().timeVectors().time() >= mettsStruct_.beta &&
 		        block[0]!=block.size()) {
 			printAdvancement(timesWithoutAdvancement_);
 			return;
 		}
 
 		if (this->common().aoe().noStageIs(StageEnumType::COLLAPSE) &&
-		        this->common().aoe().time() >= mettsStruct_.beta) {
+		        this->common().aoe().timeVectors().time() >= mettsStruct_.beta) {
 			this->common().setAllStagesTo(StageEnumType::COLLAPSE);
 			sitesCollapsed_.clear();
-			SizeType n1 = mettsStruct_.timeSteps();
-			this->common().aoe().targetVectors(n1).clear();
+			SizeType n1 = mettsStruct_.times().size();
+			this->common().aoeNonConst().destroyPvector(n1);
 			timesWithoutAdvancement_ = 0;
 			printAdvancement(timesWithoutAdvancement_);
 			return;
@@ -463,8 +467,8 @@ private:
 	                  const ProgramGlobals::DirectionEnum,
 	                  const VectorSizeType& block)
 	{
-		if (this->common().aoe().targetVectors()[index].size()==0) return;
-		assert(norm(this->common().aoe().targetVectors()[index])>1e-6);
+		if (this->tv(index).size() == 0) return;
+		assert(norm(this->tv(index)) > 1e-6);
 		VectorSizeType nk;
 		mettsCollapse_.setNk(nk,block);
 
@@ -476,7 +480,7 @@ private:
 
 			if (this->common().aoe().allStages(StageEnumType::WFT_ADVANCE)) {
 				advance = indexAdvance;
-				this->common().aoe().timeHasAdvanced();
+				this->common().aoeNonConst().timeHasAdvanced();
 			}
 
 			// don't advance the collapsed vector because we'll recompute
@@ -488,14 +492,14 @@ private:
 
 			VectorWithOffsetType phiNew; // same sectors as g.s.
 			//phiNew.populateSectors(lrs_.super());
-			assert(norm(this->common().aoe().targetVectors()[advance])>1e-6);
+			assert(norm(this->tv(advance))>1e-6);
 
 			phiNew.populateSectors(lrs_.super());
 			// OK, now that we got the partition number right, let's wft:
-			wft_.setInitialVector(phiNew,this->common().aoe().targetVectors()[advance],lrs_,nk);
+			wft_.setInitialVector(phiNew,this->tv(advance),lrs_,nk);
 			phiNew.collapseSectors();
 			assert(norm(phiNew)>1e-6);
-			this->common().aoe().targetVectors(index) = phiNew;
+			this->tvNonConst(index) = phiNew;
 		} else {
 			assert(false);
 		}
@@ -573,8 +577,8 @@ private:
 		           transformEnviron,
 		           block2);
 		pureVectors_.second = newVector2;
-		setFromInfinite(this->common().aoe().targetVectors(0),lrs_);
-		assert(norm(this->common().aoe().targetVectors()[0])>1e-6);
+		setFromInfinite(this->tvNonConst(0), lrs_);
+		assert(norm(this->tv(0)) > 1e-6);
 
 		systemPrev_.fixed = alphaFixedVolume;
 		systemPrev_.permutationInverse = lrs_.left().permutationInverse();
@@ -773,8 +777,8 @@ private:
 
 	void printEnergies() const
 	{
-		for (SizeType i=0;i<this->common().aoe().targetVectors().size();i++)
-			printEnergies(this->common().aoe().targetVectors()[i],i);
+		for (SizeType i=0;i<this->common().aoe().tvs();i++)
+			printEnergies(this->tv(i), i);
 	}
 
 	void printEnergies(const VectorWithOffsetType& phi,SizeType whatTarget) const
@@ -793,7 +797,7 @@ private:
 		typename ModelHelperType::Aux aux(p, BaseType::lrs());
 		typename ModelType::HamiltonianConnectionType hc(BaseType::lrs(),
 		                                                 BaseType::ModelType::modelLinks(),
-		                                                 this->common().aoe().time(),
+		                                                 this->common().aoe().timeVectors().time(),
 		                                                 model_.superOpHelper());
 		typename LanczosSolverType::MatrixType lanczosHelper(BaseType::model(),
 		                                                     hc,
@@ -806,7 +810,7 @@ private:
 		lanczosHelper.matrixVectorProduct(x,phi2);
 		PsimagLite::OstringStream msgg(std::cout.precision());
 		PsimagLite::OstringStream::OstringStreamType& msg = msgg();
-		msg<<"Hamiltonian average at time="<<this->common().aoe().time();
+		msg<<"Hamiltonian average at time="<<this->common().aoe().timeVectors().time();
 		msg<<" for target="<<whatTarget;
 		ComplexOrRealType numerator = phi2*x;
 		ComplexOrRealType den = phi2*phi2;
@@ -837,9 +841,9 @@ private:
 
 			weight_[i] = 0;
 
-			if (i >= this->common().aoe().targetVectors().size()) continue;
+			if (i >= this->common().aoe().tvs()) continue;
 
-			const RealType norma = norm(this->common().aoe().targetVectors()[i]);
+			const RealType norma = norm(this->tv(i));
 			if (norma < 1e-5) continue;
 
 			weight_[i] = 1;
@@ -859,7 +863,6 @@ private:
 	const WaveFunctionTransfType& wft_;
 	const QnType& quantumSector_;
 	PsimagLite::ProgressIndicator progress_;
-	VectorRealType betas_;
 	VectorRealType weight_;
 	RealType gsWeight_;
 	MettsStochasticsType mettsStochastics_;
