@@ -81,7 +81,7 @@ DISCLOSED WOULD NOT INFRINGE PRIVATELY OWNED RIGHTS.
  * We read from static tv[i] --> tv[i]
  *
  * the correction vectors are imag  --> tv[8]
- *                            real  --> tv[9]
+ *                     real  --> tv[9]
  *
  */
 
@@ -110,6 +110,7 @@ public:
 
 	typedef typename BaseType::TargetingCommonType TargetingCommonType;
 	typedef typename BaseType::MatrixVectorType MatrixVectorType;
+	typedef typename BaseType::CheckpointType CheckpointType;
 	typedef typename MatrixVectorType::ModelType ModelType;
 	typedef typename ModelType::RealType RealType;
 	typedef typename PsimagLite::Vector<RealType>::Type VectorRealType;
@@ -152,18 +153,18 @@ public:
 	typedef typename TargetingCommonType::StageEnumType StageEnumType;
 
 	TargetingRixsDynamic(const LeftRightSuperType& lrs,
-	                     const ModelType& model,
+	                     const CheckpointType& checkPoint,
 	                     const WaveFunctionTransfType& wft,
 	                     const QnType&,
 	                     InputValidatorType& ioIn)
-	    : BaseType(lrs,model,wft,1),
-	      tstStruct_(ioIn, "TargetingRixsDynamic", model),
+	    : BaseType(lrs,checkPoint,wft,1),
+	      tstStruct_(ioIn, "TargetingRixsDynamic", checkPoint.model()),
 	      tstStruct2_(nullptr),
 	      ioIn_(ioIn),
 	      progress_("TargetingRixsDynamic"),
-	      gsWeight_(1.0),
+	      gsWeight_(tstStruct_.gsWeight()),
 	      paramsForSolver_(ioIn,"DynamicDmrg"),
-	      skeleton_(ioIn_,tstStruct_,model,lrs,this->common().aoe().energy()),
+	      skeleton_(ioIn_, tstStruct_, checkPoint.model(), lrs, this->common().aoe().energy()),
 	      applied_(false),
 	      appliedFirst_(false)
 	{
@@ -176,21 +177,19 @@ public:
 			return; // early exit here
 		}
 
-		tstStruct2_ = new TargetParams2Type(ioIn, "TargetingRixsDynamic", model);
-
-		times_.resize(tstStruct2_->timeSteps());
+		tstStruct2_ = new TargetParams2Type(ioIn, "TargetingRixsDynamic", checkPoint.model());
 
 		RealType tau = tstStruct2_->tau();
-		SizeType n = times_.size();
+		SizeType n = tstStruct2_->times().size();
 		if (tstStruct_.algorithm() == TargetParamsType::BaseType::AlgorithmEnum::KRYLOVTIME) {
 			if (n != 5)
 				err("TargetingRixsDynamic with KrylovTime: number of TimeSteps must be 5\n");
 		}
 
 		for (SizeType i = 0; i < n; ++i)
-			times_[i] = i*tau/(n - 1);
+			tstStruct2_->times()[i] = i*tau/(n - 1);
 
-		this->common().aoe().initTimeVectors(*tstStruct2_, times_, ioIn);
+		this->common().aoeNonConst().initTimeVectors(*tstStruct2_, ioIn);
 	}
 
 	~TargetingRixsDynamic()
@@ -223,7 +222,7 @@ public:
 
 	RealType gsWeight() const
 	{
-		return gsWeight_;
+		return gsWeightActual_;
 	}
 
 	SizeType size() const
@@ -274,14 +273,18 @@ public:
 
 		SizeType site = block1[0];
 
-		this->common().aoe().wftSome(site, 0, 6);
+		//SizeType numberOfSites = this->lrs().super().block().size();
+		//		bool weAreAtBorder = site==0 || site==numberOfSites-1;
+		//		if (!weAreAtBorder)
+		this->common().aoeNonConst().wftSome(site, 0, 6);
 
 		const AlgorithmEnumType algo = tstStruct_.algorithm();
 		if (algo == TargetParamsType::BaseType::AlgorithmEnum::KRYLOV) {
-			this->common().aoe().wftSome(site, 6, this->common().aoe().targetVectors().size());
+//			if (!weAreAtBorder)
+				this->common().aoeNonConst().wftSome(site, 6, this->common().aoe().tvs());
 		} else {
 			// just to set the stage and currenttime: CHEBY and KRYLOVTIME
-			this->common().aoe().getPhi(0, Eg, direction, site, loopNumber, *tstStruct2_);
+			this->common().aoeNonConst().getPhi(0, Eg, direction, site, loopNumber, *tstStruct2_);
 		}
 
 		if (!applied_) {
@@ -300,6 +303,21 @@ public:
 		cocoon(site, direction);
 
 		this->common().printNormsAndWeights(gsWeight_, weight_);
+
+		//corner case
+		/*
+		SizeType site2 = numberOfSites;
+
+		if (site == 1 && direction == ProgramGlobals::DirectionEnum::EXPAND_ENVIRON)
+			site2 = 0;
+		if (site == numberOfSites - 2 &&
+		        direction == ProgramGlobals::DirectionEnum::EXPAND_SYSTEM)
+			site2 = numberOfSites - 1;
+		if (site2 == numberOfSites) return;
+		BlockType block(1, site2);
+		evolve(energies, direction, block, block2, loopNumber);
+		*/
+		setWeights();
 	}
 
 	void write(const VectorSizeType& block,
@@ -320,7 +338,9 @@ public:
 			err("TargetingRixsDynamic: number of TVs must be 6\n");
 
 		for (SizeType site = 0; site < 6; ++site)
-			this->common().aoe().targetVectors(site) = ts.vector(site);
+			this->tvNonConst(site) = ts.vector(site);
+
+		setWeights();
 	}
 
 private:
@@ -354,7 +374,7 @@ private:
 			           indexOfOperator,
 			           site,
 			           tmpV1, // phiNew
-			           this->common().aoe().targetVectors(1), // src1 apply op on Im|alpha(C)>
+			           this->tv(1), // src1 apply op on Im|alpha(C)>
 			           direction);
 
 			const VectorWithOffsetType& psi00 = this->common().aoe().
@@ -363,22 +383,22 @@ private:
 				addFactor(tmpV1, psi00, densCim);
 
 			if (tmpV1.size() > 0)
-				this->common().aoe().targetVectors(6) = tmpV1;
+				this->tvNonConst(6) = tmpV1;
 
 			VectorWithOffsetType tmpV2;
 
 			applyOneOp(loopNumber,
 			           indexOfOperator,
 			           site,
-			           tmpV2,                            // phiNew
-			           this->common().aoe().targetVectors(2), // src1 apply op on Re|alpha(C)>
+			           tmpV2,                            // phi
+			           this->tv(2), // src1 apply op on Re|alpha(C)>
 			           direction);
 
 			if (tmpV2.size() > 0)
 				addFactor(tmpV2, psi00, densCre);
 
 			if (tmpV2.size() > 0) {
-				this->common().aoe().targetVectors(7) = tmpV2;
+				this->tvNonConst(7) = tmpV2;
 				applied_ = true;
 				PsimagLite::OstringStream msgg(std::cout.precision());
 				PsimagLite::OstringStream::OstringStreamType& msg = msgg();
@@ -389,8 +409,8 @@ private:
 	}
 
 	void doMax2Sum(SizeType site,
-	            ProgramGlobals::DirectionEnum direction,
-	            SizeType loopNumber)
+	               ProgramGlobals::DirectionEnum direction,
+	               SizeType loopNumber)
 	{
 
 		if (site == tstStruct_.sites(0)) {
@@ -400,22 +420,22 @@ private:
 			           indexOfOperator,
 			           site,
 			           tmpV1, // phiNew
-			           this->common().aoe().targetVectors(1), // src1 apply op on Im|alpha(C)>
+			           this->tv(1), // src1 apply op on Im|alpha(C)>
 			           direction);
 
 			if (tmpV1.size() > 0)
-				this->common().aoe().targetVectors(6) = tmpV1;
+				this->tvNonConst(6) = tmpV1;
 
 			VectorWithOffsetType tmpV2;
 			applyOneOp(loopNumber,
 			           indexOfOperator,
 			           site,
 			           tmpV2,                            // phiNew
-			           this->common().aoe().targetVectors(2), // src1 apply op on Re|alpha(C)>
+			           this->tv(2), // src1 apply op on Re|alpha(C)>
 			           direction);
 
 			if (tmpV2.size() > 0) {
-				this->common().aoe().targetVectors(7) = tmpV2;
+				this->tvNonConst(7) = tmpV2;
 				applied_ = false;
 				appliedFirst_ = true;
 				PsimagLite::OstringStream msgg(std::cout.precision());
@@ -449,7 +469,7 @@ private:
 			           indexOfOperator,
 			           site,
 			           tmpV1, // phiNew
-			           this->common().aoe().targetVectors(1), // src1 apply op on Im|alpha(C)>
+			           this->tv(1), // src1 apply op on Im|alpha(C)>
 			           direction);
 
 			const VectorWithOffsetType& psi00 = this->common().aoe().
@@ -459,21 +479,21 @@ private:
 				addFactor(tmpV1, psi00, densCim);
 
 			if (tmpV1.size() > 0)
-				this->common().aoe().targetVectors(6) += tmpV1;
+				this->tvNonConst(6) += tmpV1;
 
 			VectorWithOffsetType tmpV2;
 			applyOneOp(loopNumber,
 			           indexOfOperator,
 			           site,
 			           tmpV2,                            // phiNew
-			           this->common().aoe().targetVectors(2), // src1 apply op on Re|alpha(C)>
+			           this->tv(2), // src1 apply op on Re|alpha(C)>
 			           direction);
 
 			if (tmpV2.size() > 0)
 				addFactor(tmpV2, psi00, densCre);
 
 			if (tmpV2.size() > 0) {
-				this->common().aoe().targetVectors(7) += tmpV2;
+				this->tvNonConst(7) += tmpV2;
 				applied_ = true;
 				PsimagLite::OstringStream msgg(std::cout.precision());
 				PsimagLite::OstringStream::OstringStreamType& msg = msgg();
@@ -484,8 +504,8 @@ private:
 	}
 
 	void doMax2Prod(SizeType site,
-	            ProgramGlobals::DirectionEnum direction,
-	            SizeType loopNumber)
+	                ProgramGlobals::DirectionEnum direction,
+	                SizeType loopNumber)
 	{
 
 		if (site == tstStruct_.sites(0)) {
@@ -495,22 +515,22 @@ private:
 			           indexOfOperator,
 			           site,
 			           tmpV1, // phiNew
-			           this->common().aoe().targetVectors(1), // src1 apply op on Im|alpha(C)>
+			           this->tv(1), // src1 apply op on Im|alpha(C)>
 			           direction);
 
 			if (tmpV1.size() > 0)
-				this->common().aoe().targetVectors(6) = tmpV1;
+				this->tvNonConst(6) = tmpV1;
 
 			VectorWithOffsetType tmpV2;
 			applyOneOp(loopNumber,
 			           indexOfOperator,
 			           site,
 			           tmpV2,                            // phiNew
-			           this->common().aoe().targetVectors(2), // src1 apply op on Re|alpha(C)>
+			           this->tv(2), // src1 apply op on Re|alpha(C)>
 			           direction);
 
 			if (tmpV2.size() > 0) {
-				this->common().aoe().targetVectors(7) = tmpV2;
+				this->tvNonConst(7) = tmpV2;
 				applied_ = false;
 				appliedFirst_ = true;
 				PsimagLite::OstringStream msgg(std::cout.precision());
@@ -528,22 +548,22 @@ private:
 			           indexOfOperator,
 			           site,
 			           tmpV1, // phiNew
-			           this->common().aoe().targetVectors(6), // src1 apply op on Im|alpha(C)>
+			           this->tv(6), // src1 apply op on Im|alpha(C)>
 			           direction);
 
 			if (tmpV1.size() > 0)
-				this->common().aoe().targetVectors(6) = tmpV1;
+				this->tvNonConst(6) = tmpV1;
 
 			VectorWithOffsetType tmpV2;
 			applyOneOp(loopNumber,
 			           indexOfOperator,
 			           site,
 			           tmpV2,                            // phiNew
-			           this->common().aoe().targetVectors(7), // src1 apply op on Re|alpha(C)>
+			           this->tv(7), // src1 apply op on Re|alpha(C)>
 			           direction);
 
 			if (tmpV2.size() > 0) {
-				this->common().aoe().targetVectors(7) = tmpV2;
+				this->tvNonConst(7) = tmpV2;
 				applied_ = true;
 				PsimagLite::OstringStream msgg(std::cout.precision());
 				PsimagLite::OstringStream::OstringStreamType& msg = msgg();
@@ -575,7 +595,10 @@ private:
 		const ComplexOrRealType ii =
 		        this->common().rixsCocoon(direction,site,eightOrEleven,4,true);
 
-		const RealType time = this->common().aoe().time();
+		RealType time = 0.0;
+		if (tstStruct_.algorithm() != TargetParamsType::BaseType::AlgorithmEnum::KRYLOV) {
+			time = this->common().aoe().timeVectors().time();
+		}
 		std::cout<<site<<" "<<(ri-ir)<<" "<<time; // time here is the currentTime
 		std::cout<<" <gs|A|P2> 1\n";   // 1 here is the "superdensity"
 		std::cout<<site<<" "<<(rr+ii)<<" "<<time; // time here is the currentTime
@@ -604,47 +627,43 @@ private:
 	                    const VectorSizeType& block1)
 	{
 		if (!applied_ && appliedFirst_) {
-			setWeights(8);
 			return;
 		}
 
 		if (!applied_) {
-			setWeights(6);
 			return;
 		}
 
 		const AlgorithmEnumType algo = tstStruct_.algorithm();
 
 		if (algo == TargetParamsType::BaseType::AlgorithmEnum::KRYLOV) {
-			skeleton_.calcDynVectors(this->common().aoe().targetVectors(6),
-			                         this->common().aoe().targetVectors(7),
-			                         this->common().aoe().targetVectors(8),
-			                         this->common().aoe().targetVectors(9));
-			setWeights(10);
+			skeleton_.calcDynVectors(this->tv(6),
+			                         this->tv(7),
+			                         this->tvNonConst(8),
+			                         this->tvNonConst(9));
 			firstCall_ = false; // unused here but just in case
 			return;
 		}
 
 		VectorSizeType indices;
 		VectorSizeType indices2;
-		SizeType numberOfWeights = 0;
+		//SizeType numberOfWeights = 0;
 
 		if (algo == TargetParamsType::BaseType::AlgorithmEnum::CHEBYSHEV) {
 			indices = {6, 8, 9};
 			indices2 = {7, 10, 11};
-			numberOfWeights = 12;
+			//numberOfWeights = 12;
 		} else if (algo == TargetParamsType::BaseType::AlgorithmEnum::KRYLOVTIME){
 			indices = {6, 8, 9, 10, 11};
 			indices2 = {7, 12, 13, 14, 15};
-			numberOfWeights = 16;
+			//numberOfWeights = 16;
 		}
 
-		assert(numberOfWeights > 0);
+		//assert(numberOfWeights > 0);
 		assert(indices.size() > 0 && indices2.size() > 0);
 		calcVectors(indices, Eg, direction, block1, !firstCall_, false);
 		calcVectors(indices2, Eg, direction, block1, !firstCall_, true);
 		firstCall_ = false;
-		setWeights(numberOfWeights);
 	}
 
 	void calcVectors(const VectorSizeType& indices,
@@ -657,16 +676,16 @@ private:
 		bool allOperatorsApplied = (this->common().aoe().noStageIs(StageEnumType::DISABLED) &&
 		                            this->common().aoe().noStageIs(StageEnumType::OPERATOR));
 
-		const VectorWithOffsetType& v0 = this->common().aoe().targetVectors(indices[0]);
+		const VectorWithOffsetType& v0 = this->tv(indices[0]);
 
-		this->common().aoe().calcTimeVectors(indices,
-		                                     Eg,
-		                                     v0,
-		                                     direction,
-		                                     allOperatorsApplied,
-		                                     wftOrAdvance, // wft and advance indices[0]
-		                                     block1,
-		                                     isLastCall);
+		this->common().aoeNonConst().calcTimeVectors(indices,
+		                                             Eg,
+		                                             v0,
+		                                             direction,
+		                                             allOperatorsApplied,
+		                                             wftOrAdvance, // wft and advance indices[0]
+		                                             block1,
+		                                             isLastCall);
 	}
 
 	void applyOneOp(SizeType loopNumber,
@@ -679,31 +698,45 @@ private:
 		const AlgorithmEnumType algo = tstStruct_.algorithm();
 
 		if (algo != TargetParamsType::BaseType::AlgorithmEnum::KRYLOV)
-			this->common().aoe().applyOneOperator(loopNumber,
-		                                          indexOfOperator,
-		                                          site,
-		                                          dest, // phiNew
-		                                          src, // src1
-		                                          direction,
-		                                          *tstStruct2_);
+			this->common().aoeNonConst().applyOneOperator(loopNumber,
+		                                                  indexOfOperator,
+		                                                  site,
+		                                                  dest, // phiNew
+		                                                  src, // src1
+		                                                  direction,
+		                                                  *tstStruct2_);
 		else
-			this->common().aoe().applyOneOperator(loopNumber,
-		                                          indexOfOperator,
-		                                          site,
-		                                          dest, // phiNew
-		                                          src, // src1
-		                                          direction,
-		                                          tstStruct_);
+			this->common().aoeNonConst().applyOneOperator(loopNumber,
+		                                                  indexOfOperator,
+		                                                  site,
+		                                                  dest, // phiNew
+		                                                  src, // src1
+		                                                  direction,
+		                                                  tstStruct_);
 	}
 
-	void setWeights(SizeType n)
+	void setWeights()
 	{
-		gsWeight_ = tstStruct_.gsWeight();
+		const SizeType n = this->numberOfTvs();
+		weight_.resize(n);
+		std::fill(weight_.begin(), weight_.end(), 0);
+		RealType sum = 0;
+		for (SizeType i = 0; i < n; ++i) {
+			RealType norma = norm(this->tv(i));
+			if (norma < 1e-6) continue;
+			weight_[i] = 1/norma;
+			sum += weight_[i];
+		}
 
-		RealType sum  = n;
-		weight_.resize(n, 1);
+		gsWeightActual_ = 1 - sum;
 
-		for (SizeType r=0;r<weight_.size();r++) weight_[r] = (1.0 - gsWeight_)/sum;
+		if (gsWeightActual_ >= gsWeight_) return; // <--- EARLY EXIT HERE
+
+		assert(sum > 1e-6);
+		RealType factor = (1 - gsWeight_)/sum;
+		for (SizeType i = 0; i < n; ++i)
+			weight_[i] *= factor;
+		gsWeightActual_ = gsWeight_;
 	}
 
 	TargetParamsType tstStruct_;
@@ -711,7 +744,7 @@ private:
 	InputValidatorType& ioIn_;
 	PsimagLite::ProgressIndicator progress_;
 	RealType gsWeight_;
-	VectorRealType times_;
+	RealType gsWeightActual_;
 	typename PsimagLite::Vector<RealType>::Type weight_;
 	typename LanczosSolverType::ParametersSolverType paramsForSolver_;
 	CorrectionVectorSkeletonType skeleton_;
