@@ -127,7 +127,7 @@ class TimeVectorsChebyshev : public  TimeVectorsBase<TargetParamsType,
 	VectorMatrixFieldType;
 	typedef typename LanczosSolverType::TridiagonalMatrixType TridiagonalMatrixType;
 	typedef typename ModelType::InputValidatorType InputValidatorType;
-	typedef typename PsimagLite::Vector<VectorWithOffsetType>::Type
+	typedef typename PsimagLite::Vector<VectorWithOffsetType*>::Type
 	VectorVectorWithOffsetType;
 	typedef typename PsimagLite::Vector<VectorRealType>::Type VectorVectorRealType;
 	typedef typename LanczosSolverType::MatrixType MatrixLanczosType;
@@ -140,29 +140,51 @@ class TimeVectorsChebyshev : public  TimeVectorsBase<TargetParamsType,
 
 public:
 
-	TimeVectorsChebyshev(const SizeType& currentTimeStep,
-	                     const TargetParamsType& tstStruct,
-	                     const VectorRealType& times,
+	TimeVectorsChebyshev(const TargetParamsType& tstStruct,
 	                     VectorVectorWithOffsetType& targetVectors,
 	                     const ModelType& model,
 	                     const WaveFunctionTransfType& wft,
 	                     const LeftRightSuperType& lrs,
 	                     InputValidatorType& ioIn)
-	    : BaseType(model, lrs, wft),
-	      currentTimeStep_(currentTimeStep),
+	    : BaseType(model, lrs, wft, "chebyshev"),
 	      tstStruct_(tstStruct),
-	      times_(times),
 	      targetVectors_(targetVectors),
 	      model_(model),
 	      wft_(wft),
 	      lrs_(lrs),
 	      ioIn_(ioIn),
 	      timeHasAdvanced_(false),
-	      correctVectorsAwesomePred_("0==1") // never correct
+	      correctVectorsAwesomePred_("0==1"), // never correct
+	      isFreqTargeted_(false),
+	      omega_(0.0),
+	      lambda_(1.0)
 	{
 		try {
 			ioIn_.readline(correctVectorsAwesomePred_, "ChebyshevCorrectVector=");
 		} catch (std::exception&) {}
+		try {
+			PsimagLite::String isFreqTargeted;
+			ioIn_.readline(isFreqTargeted, "CorrectionVectorTargeted=");
+			if (isFreqTargeted=="true"||isFreqTargeted=="True")
+				isFreqTargeted_ = true;
+		} catch (std::exception&) {}
+
+		if (isFreqTargeted_) {
+			PsimagLite::String warning("TargetingChebyshev: ");
+			warning += "Frequency omega targeted!\n";
+			std::cout<<warning;
+			std::cerr<<warning;
+			SizeType n = targetVectors_.size();
+			if (n != 4) {
+				PsimagLite::String msg = "TimeVectorsChebyshevFrequency:" +
+				        PsimagLite::String("number of targets must be 4\n");
+				std::cout<<msg;
+				std::cerr<<msg;
+				return;
+			}
+			ioIn_.readline(omega_, "CorrectionVectorOmega=");
+			ioIn_.readline(lambda_, "CorrectionVectorLambda=");
+		}
 	}
 
 	virtual void calcTimeVectors(const VectorSizeType& indices,
@@ -170,25 +192,28 @@ public:
 	                             const VectorWithOffsetType& phi,
 	                             const typename BaseType::ExtraData& extra)
 	{
+		const VectorRealType& times = tstStruct_.times();
+
 		if (extra.wftAndAdvanceIfNeeded) {
 			const SizeType noAdvance = indices[0];
 			VectorWithOffsetType phiNew;
-			if (targetVectors_[noAdvance].size() > 0) {
+			assert(targetVectors_[noAdvance]);
+			if (targetVectors_[noAdvance]->size() > 0) {
 				BaseType::wftHelper().wftOneVector(phiNew,
-				                                   targetVectors_[noAdvance],
+				                                   *targetVectors_[noAdvance],
 				                                   extra.block[0]);
 
-				targetVectors_[noAdvance] = phiNew;
+				*targetVectors_[noAdvance] = phiNew;
 			}
 		}
 
 		SizeType startOfWft = 1;
-		if (currentTimeStep_ == 0) {
+		if (this->currentTimeStep() == 0) {
 			SizeType indexOf1 = indices[startOfWft];
 			assert(indexOf1 < targetVectors_.size());
-			VectorWithOffsetType& tv1 =
-			        const_cast<VectorWithOffsetType&>(targetVectors_[indexOf1]);
-			tv1  = phi;
+			VectorWithOffsetType* tv1 =
+			        const_cast<VectorWithOffsetType*>(targetVectors_[indexOf1]);
+			*tv1  = phi;
 			startOfWft = 2;
 		}
 
@@ -197,44 +222,59 @@ public:
 		assert(0 < extra.block.size());
 		SizeType n = indices.size();
 
-		for (SizeType i = startOfWft; i < n; ++i) {
+		SizeType m = indices.size()-1;
+
+		if (!isFreqTargeted_)
+			m = n;
+
+		for (SizeType i = startOfWft; i < m; ++i) {
 			SizeType ii = indices[i];
 			BaseType::wftHelper().wftSome(targetVectors_, extra.block[0], ii, ii + 1);
 		}
 
+		if (isFreqTargeted_)
+			BaseType::wftHelper().wftSome(targetVectors_, extra.block[0], m, n);
+
 		assert(n > 0);
-		if (currentTimeStep_ == 0 && tstStruct_.noOperator() && tstStruct_.skipTimeZero()) {
+		if (this->currentTimeStep() == 0 && tstStruct_.noOperator() && tstStruct_.skipTimeZero()) {
 			for (SizeType i = 0; i < n; ++i) {
 				SizeType ii = indices[i];
-				targetVectors_[ii] = phi;
+				*targetVectors_[ii] = phi;
 			}
 
 			return;
 		}
 
-		const VectorWithOffsetType* ptr0 = &(targetVectors_[indices[0]]);
+		const VectorWithOffsetType* ptr0 = targetVectors_[indices[0]];
 		const VectorWithOffsetType* ptr1 = &phi;
 		if (ptr0 != ptr1)
-			targetVectors_[indices[0]] = phi;
+			*targetVectors_[indices[0]] = phi;
 
-		if (times_.size() == 1 && fabs(times_[0])<1e-10) return;
+		if (times.size() == 1 && fabs(times[0])<1e-10) return;
 
 		if (timeHasAdvanced_) {
-			for (SizeType i = 0; i < n - 1; ++i) {
+			for (SizeType i = 0; i < m - 1; ++i) {
 				SizeType ii = indices[i];
 				SizeType jj = indices[i + 1];
-				targetVectors_[ii] = targetVectors_[jj];
+				*targetVectors_[ii] = *targetVectors_[jj];
 			}
 		}
 
-		for (SizeType i = 2; i < n; ++i) {
+		for (SizeType i = 2; i < m; ++i) {
 			SizeType ii = indices[i];
 			assert(ii < targetVectors_.size());
 			assert(ii != 1);
-			targetVectors_[ii] = phi;
+			*targetVectors_[ii] = phi;
 			SizeType prev = indices[i - 1];
 			SizeType prevMinus2 = indices[i - 2];
-			calcTargetVector(targetVectors_[ii], phi, prev, prevMinus2, Eg, extra.time);
+			calcTargetVector(*targetVectors_[ii], phi, prev, prevMinus2, Eg);
+			if (isFreqTargeted_) {
+				SizeType post = indices[i+1];
+				VectorWithOffsetType phiNew = phi;
+				calcCorrectionVector(phiNew, *targetVectors_[ii],
+				                     prev, prevMinus2, Eg);
+				*targetVectors_[post] += phiNew;
+			}
 		}
 
 		assert(extra.block.size() > 0);
@@ -244,15 +284,15 @@ public:
 		PredicateAwesomeType pred(correctVectorsAwesomePred_);
 		const bool flagcorrection = pred.isTrue("%s", site, "%c", center, "%n", nsites);
 		if (flagcorrection)
-			correctVectors(indices, Eg, extra.time);
+			correctVectors(indices, Eg);
 
 		timeHasAdvanced_ = false;
 	}
 
-	void timeHasAdvanced(RealType& time)
+	void timeHasAdvanced()
 	{
 		timeHasAdvanced_ = true;
-		time += tstStruct_.tau();
+		this->advanceCurrentTime(tstStruct_.tau());
 	}
 
 private:
@@ -261,14 +301,13 @@ private:
 	                      const VectorWithOffsetType& phi,
 	                      SizeType prev,
 	                      SizeType prevMinus2,
-	                      RealType Eg,
-	                      RealType currentTime)
+	                      RealType Eg)
 
 	{
 		for (SizeType ii=0;ii<phi.sectors();ii++) {
 			SizeType i0 = phi.sector(ii);
 			TargetVectorType r;
-			calcTargetVector(r, phi, prev, prevMinus2, i0, Eg, currentTime);
+			calcTargetVector(r, phi, prev, prevMinus2, i0, Eg);
 			v.setDataInSector(r,i0);
 		}
 	}
@@ -278,14 +317,13 @@ private:
 	                      SizeType prev,
 	                      SizeType prevMinus2,
 	                      SizeType i0,
-	                      RealType Eg,
-	                      RealType currentTime)
+	                      RealType Eg)
 	{
 		SizeType p = lrs_.super().findPartitionNumber(phi.offset(i0));
 		typename ModelHelperType::Aux aux(p, lrs_);
 		typename ModelType::HamiltonianConnectionType hc(lrs_,
 		                                                 ModelType::modelLinks(),
-		                                                 currentTime,
+		                                                 this->time(),
 		                                                 model_.superOpHelper());
 		MatrixLanczosType lanczosHelper(model_, hc, aux);
 
@@ -299,20 +337,86 @@ private:
 		SizeType total = phi.effectiveSize(i0);
 		TargetVectorType phi2(total);
 		r.resize(total);
-		if (currentTimeStep_ == 0) {
+		if (this->currentTimeStep() == 0) {
 			phi.extract(phi2,i0);
 			lanczosHelper2.matrixVectorProduct(r,phi2); // applying Hprime
 		} else {
 			TargetVectorType x2(total);
-			VectorWithOffsetType x = 2.0*targetVectors_[prev];
+			VectorWithOffsetType x = 2.0*(*targetVectors_[prev]);
 			x.extract(x2,i0);
-			targetVectors_[prevMinus2].extract(phi2,i0);
+			targetVectors_[prevMinus2]->extract(phi2,i0);
 			lanczosHelper2.matrixVectorProduct(r,x2); // applying Hprime
 			r += (-1.0)*phi2;
 		}
 	}
 
-	void correctVectors(const VectorSizeType& indices, RealType Eg, RealType currentTime)
+	void calcCorrectionVector(VectorWithOffsetType& v,
+	                          const VectorWithOffsetType& phi,
+	                          SizeType prev,
+	                          SizeType prevMinus2,
+	                          RealType Eg)
+
+	{
+		for (SizeType ii=0;ii<phi.sectors();ii++) {
+			SizeType i0 = phi.sector(ii);
+			TargetVectorType r;
+			calcCorrectionVector(r, phi, prev, prevMinus2, i0);
+			v.setDataInSector(r,i0);
+		}
+	}
+	void calcCorrectionVector(TargetVectorType& r,
+	                          const VectorWithOffsetType& phi,
+	                          SizeType prev,
+	                          SizeType prevMinus2,
+	                          SizeType i0)
+	{
+		SizeType total = phi.effectiveSize(i0);
+		TargetVectorType phiAdd(total);
+		r.resize(total);
+
+		RealType c = tstStruct_.chebyTransform()[0];
+
+		if (this->currentTimeStep()==0) {
+
+			RealType factor = computeChebyAndDamping(2,omega_);
+			RealType omegaden = sqrt(1.0-omega_*omega_);
+
+			RealType f = 2.0*c*factor/omegaden;
+			VectorWithOffsetType phi_temp = f*(phi);
+			phi_temp.extract(phiAdd,i0);
+			r += phiAdd;
+
+			factor = computeChebyAndDamping(1,omega_);
+			f = 2.0*c*factor/omegaden;
+			phi_temp = f*(*targetVectors_[prev]);
+			phi_temp.extract(phiAdd,i0);
+			r += phiAdd;
+
+			factor = computeChebyAndDamping(0,omega_);
+			f = c*factor/omegaden;
+			phi_temp = f*(*targetVectors_[prevMinus2]);
+			phi_temp.extract(phiAdd,i0);
+			r += phiAdd;
+		} else {
+			double time = 2+this->currentTimeStep();
+			RealType factor = computeChebyAndDamping(time,omega_);
+			RealType omegaden = sqrt(1.0-omega_*omega_);
+			RealType f = 2.0*c*factor/omegaden;
+			VectorWithOffsetType phi_temp = f*(phi);
+			phi_temp.extract(phiAdd,i0);
+			r += phiAdd;
+		}
+	}
+
+	RealType computeChebyAndDamping(SizeType index, RealType omega)
+	{
+		RealType ChebyPoly = cos(index*acos(omega));
+		RealType Ntotal = 1.0*model_.params().finiteLoop.size()-2.0;
+		RealType DampingFactor = sinh(lambda_*(1.0-index/Ntotal))/sinh(lambda_);
+		return ChebyPoly*DampingFactor;
+	}
+
+	void correctVectors(const VectorSizeType& indices, RealType Eg)
 	{
 		// take the first vector and compute V and weights
 		const SizeType n = indices.size();
@@ -326,7 +430,8 @@ private:
 		}
 
 		const SizeType indexToUse = 0;
-		const VectorWithOffsetType& phi = targetVectors_[indices[indexToUse]];
+		assert(targetVectors_[indices[indexToUse]]);
+		const VectorWithOffsetType& phi = *targetVectors_[indices[indexToUse]];
 
 		if (phi.sectors() == 0) {
 			PsimagLite::String msg = "TimeVectorsChebyshev:: correctVectors " +
@@ -346,14 +451,14 @@ private:
 
 		VectorSizeType permutation;
 		computeAuxForCorrection(V, T, weights, permutation,
-		                        phi, phi.sector(0), Eg, currentTime);
+		                        phi, phi.sector(0), Eg, this->time());
 		if (weights.size() == 0) return;
 		assert(V.size() == 1);
 
 		// correct all vectors based on the same V and weights
 		const SizeType nvectors = indices.size(); // = 3
 		for (SizeType i = 0; i < nvectors; ++i)
-			correctVectors(targetVectors_[indices[i]], V[0], T[0], weights, permutation);
+			correctVectors(*targetVectors_[indices[i]], V[0], T[0], weights, permutation);
 	}
 
 	void correctVectors(VectorWithOffsetType& phi,
@@ -485,9 +590,7 @@ private:
 			r[i] *= factor;
 	}
 
-	const SizeType& currentTimeStep_;
 	const TargetParamsType& tstStruct_;
-	const VectorRealType& times_;
 	VectorVectorWithOffsetType& targetVectors_;
 	const ModelType& model_;
 	const WaveFunctionTransfType& wft_;
@@ -495,6 +598,9 @@ private:
 	InputValidatorType& ioIn_;
 	bool timeHasAdvanced_;
 	PsimagLite::String correctVectorsAwesomePred_;
+	bool isFreqTargeted_;
+	RealType omega_;
+	RealType lambda_;
 }; //class TimeVectorsChebyshev
 } // namespace Dmrg
 /*@}*/
