@@ -83,6 +83,7 @@ DISCLOSED WOULD NOT INFRINGE PRIVATELY OWNED RIGHTS.
 #include "MatrixOrIdentity.h"
 #include "Sort.h"
 #include "Utils.h"
+#include "PackIndices.h"
 
 namespace Dmrg {
 
@@ -119,25 +120,21 @@ class TimeVectorsSuzukiTrotter : public  TimeVectorsBase<
 	typedef typename PsimagLite::Vector<RealType>::Type VectorRealType;
 	typedef VectorComplexOrRealType TargetVectorType;
 	typedef PsimagLite::Vector<SizeType>::Type VectorSizeType;
-	typedef typename PsimagLite::Vector<VectorWithOffsetType>::Type
+	typedef typename PsimagLite::Vector<VectorWithOffsetType*>::Type
 	VectorVectorWithOffsetType;
 	typedef typename ModelType::HilbertBasisType HilbertBasisType;
 	typedef typename ModelType::HilbertBasisType::value_type HilbertStateType;
 
 public:
 
-	TimeVectorsSuzukiTrotter(const SizeType& currentTimeStep,
-	                         const TargetParamsType& tstStruct,
-	                         const VectorRealType& times,
+	TimeVectorsSuzukiTrotter(const TargetParamsType& tstStruct,
 	                         VectorVectorWithOffsetType& targetVectors,
 	                         const ModelType& model,
 	                         const WaveFunctionTransfType& wft,
 	                         const LeftRightSuperType& lrs)
-	    : BaseType(model, lrs, wft),
+	    : BaseType(model, lrs, wft, "suzukitrotter"),
 	      progress_("TimeVectorsSuzukiTrotter"),
-	      currentTimeStep_(currentTimeStep),
 	      tstStruct_(tstStruct),
-	      times_(times),
 	      targetVectors_(targetVectors),
 	      model_(model),
 	      wft_(wft),
@@ -160,17 +157,17 @@ public:
 		progress_.printline(msgg, std::cout);
 
 		// set non-zero sectors
-		targetVectors_[0] = phi;
+		*targetVectors_[0] = phi;
 
 		bool returnFlag = false;
-		if (times_.size() != indices.size())
+		if (tstStruct_.times().size() != indices.size())
 			err("TimeVectorsSuzukiTrotter: times.size() != indices.size()\n");
 
-		for (SizeType i = 1; i < times_.size(); ++i) {
+		for (SizeType i = 1; i < tstStruct_.times().size(); ++i) {
 			const SizeType ii = indices[i];
 			assert(ii < targetVectors_.size());
-			if (targetVectors_[ii].size()==0 || !extraData.allOperatorsApplied) {
-				targetVectors_[ii] = phi;
+			if (targetVectors_[ii]->size() == 0 || !extraData.allOperatorsApplied) {
+				*targetVectors_[ii] = phi;
 				returnFlag = true;
 			}
 		}
@@ -242,22 +239,22 @@ public:
 		for (SizeType i = 1; i < n; ++i) {
 			const SizeType ii = indices[i];
 			assert(ii < targetVectors_.size());
-			VectorWithOffsetType src = targetVectors_[ii];
-			// Only time differences here (i.e. times_[i] not times_[i]+currentTime_)
-			calcTargetVector(targetVectors_[ii],
+			VectorWithOffsetType src = *targetVectors_[ii];
+			// Only time differences here (i.e. extra.times[i] not extra.times[i]+currentTime_)
+			calcTargetVector(*targetVectors_[ii],
 			                 Eg,
 			                 src,
 			                 extraData.dir,
-			                 times_[i],
+			                 tstStruct_.times()[i],
 			                 transformS,
 			                 transformST,
 			                 transformE,
 			                 transformET);
-			assert(targetVectors_[ii].size()==targetVectors_[indices[0]].size());
+			assert(targetVectors_[ii]->size() == targetVectors_[indices[0]]->size());
 		}
 	}
 
-	virtual void timeHasAdvanced(RealType&)
+	void timeHasAdvanced()
 	{
 		linksSeen_.clear();
 		PsimagLite::OstringStream msgg(std::cout.precision());
@@ -284,22 +281,22 @@ private:
 
 	void wftAll(const VectorSizeType& block)
 	{
-		for (SizeType i=1;i<times_.size();i++)
+		for (SizeType i=1;i<tstStruct_.times().size();i++)
 			wftOne(i,block);
 	}
 
 	void wftOne(SizeType i,const VectorSizeType& block)
 	{
-		VectorWithOffsetType phiNew = targetVectors_[0];
+		VectorWithOffsetType phiNew = *targetVectors_[0];
 
 		// OK, now that we got the partition number right, let's wft:
 		VectorSizeType nk;
 		setNk(nk,block);
 		// generalize for su(2)
-		wft_.setInitialVector(phiNew,targetVectors_[i],lrs_,nk);
+		wft_.setInitialVector(phiNew,*targetVectors_[i],lrs_,nk);
 		phiNew.collapseSectors();
 		assert(norm(phiNew)>1e-6);
-		targetVectors_[i]=phiNew;
+		*targetVectors_[i]=phiNew;
 	}
 
 	void calcTargetVector(VectorWithOffsetType& target,
@@ -586,9 +583,7 @@ private:
 	}
 
 	PsimagLite::ProgressIndicator progress_;
-	const SizeType& currentTimeStep_;
 	const TargetParamsType& tstStruct_;
-	const VectorRealType& times_;
 	VectorVectorWithOffsetType& targetVectors_;
 	const ModelType& model_;
 	const WaveFunctionTransfType& wft_;
