@@ -129,6 +129,7 @@ public:
 	typedef typename ModelBaseType::OpsLabelType OpsLabelType;
 	typedef typename ModelBaseType::OpForLinkType OpForLinkType;
 	typedef Aklt<ModelBaseType> AkltType;
+	using ModelParametersType = ParametersModelHeisenberg<RealType, QnType>;
 
 	ModelHeisenberg(const SolverParamsType& solverParams,
 	                InputValidatorType& io,
@@ -144,15 +145,12 @@ public:
 	      aklt_(*this, additional)
 	{
 		SizeType n = superGeometry_.numberOfSites();
-		SizeType m = modelParameters_.magneticFieldV.size();
 		SizeType md = modelParameters_.anisotropyD.size();
 		SizeType me = modelParameters_.anisotropyE.size();
 
-		if (m > 0 && m != n) {
-			PsimagLite::String msg("ModelHeisenberg: If provided, ");
-			msg += " MagneticField must be a vector of " + ttos(n) + " entries.\n";
-			err(msg);
-		}
+		ModelParametersType::checkMagneticField(modelParameters_.magneticFieldX.size(), 'X', n);
+
+		ModelParametersType::checkMagneticField(modelParameters_.magneticFieldZ.size(), 'Z', n);
 
 		if (md > 0 && md != n) {
 			PsimagLite::String msg("ModelHeisenberg: If provided, ");
@@ -165,12 +163,6 @@ public:
 			msg += " AnisotropyE must be a vector of " + ttos(n) + " entries.\n";
 			throw PsimagLite::RuntimeError(msg);
 		}
-
-		if (BasisType::useSu2Symmetry() && modelParameters_.twiceTheSpin != 1) {
-			PsimagLite::String msg("ModelHeisenberg: SU(2) symmetry, ");
-			msg += " for spin different than 1/2 is not implemented yet.\n";
-			throw PsimagLite::RuntimeError(msg);
-		}
 	}
 
 	void write(PsimagLite::String label1, PsimagLite::IoNg::Out::Serializer& io) const
@@ -193,8 +185,10 @@ public:
 	 */
 	void addDiagonalsInNaturalBasis(SparseMatrixType &hmatrix,
 	                                const BlockType& block,
-	                                RealType) const
+	                                RealType time) const
 	{
+		ModelBaseType::additionalOnSiteHamiltonian(hmatrix, block, time);
+
 		SizeType linSize = superGeometry_.numberOfSites();
 		SizeType n = block.size();
 
@@ -206,19 +200,8 @@ public:
 			const OperatorType& sz = ModelBaseType::naturalOperator("sz", site, 0);
 			const OperatorType& splus = ModelBaseType::naturalOperator("splus", site, 0);
 
-			if (modelParameters_.magneticFieldV.size() == linSize) {
-
-				RealType tmp = modelParameters_.magneticFieldV[site];
-				const OperatorType& sminus = ModelBaseType::naturalOperator("sminus", site, 0);
-
-				if (modelParameters_.magneticFieldDirection == "z") {
-					hmatrix += tmp*sz.getCRS();
-				} else if (modelParameters_.magneticFieldDirection == "x") {
-					static const RealType zeroPointFive = 0.5;
-					hmatrix += zeroPointFive*tmp*splus.getCRS();
-					hmatrix += zeroPointFive*tmp*sminus.getCRS();
-				}
-			}
+			addMagneticField(hmatrix, 'X', site);
+			addMagneticField(hmatrix, 'Z', site);
 
 			// PSIDOCMARK_END
 
@@ -278,7 +261,8 @@ protected:
 			                  -1,
 			                  su2related);
 			this->createOpsLabel("splus").push(myOp);
-			this->makeTrackable("splus");
+			if (additional_ != "2")
+				this->makeTrackable("splus");
 
 			myOp.dagger();
 			this->createOpsLabel("sminus").push(myOp);
@@ -295,7 +279,7 @@ protected:
 			this->makeTrackable("sz");
 
 			// Set the operators S^x_i in the natural basis
-			tmpMatrix = findSxMatrices(i,natBasis);
+			tmpMatrix = findSxOrSyBarMatrices(i,natBasis, "sx");
 			typename OperatorType::Su2RelatedType su2related3;
 			OperatorType myOp3(tmpMatrix,
 			                   ProgramGlobals::FermionOrBosonEnum::BOSON,
@@ -304,9 +288,21 @@ protected:
 			                   su2related3);
 			this->createOpsLabel("sx").push(myOp3);
 
-			if (additional_ == "Anisotropic")
+			if (additional_ == "Anisotropic" || additional_ == "2")
 				this->makeTrackable("sx");
 
+			// Set the operators S^ybar_i in the natural basis
+			tmpMatrix = findSxOrSyBarMatrices(i, natBasis, "sybar");
+			OperatorType syBarOp(tmpMatrix,
+			                     ProgramGlobals::FermionOrBosonEnum::BOSON,
+			                     PairType(0, 0),
+			                     1,
+			                     su2related3);
+			this->createOpsLabel("sybar").push(syBarOp);
+
+			if (additional_ == "2")
+				this->makeTrackable("sybar");
+
 			aklt_.fillLabeledOperators(i, myOp.getCRS(), myOp2.getCRS());
 
 			tmpMatrix = findMaximal(i, natBasis);
@@ -321,29 +317,24 @@ protected:
 
 	void fillModelLinks()
 	{
-		bool isSu2 = BasisType::useSu2Symmetry();
+		if (additional_ == "2") {
+			fillModelLinks2();
+			return;
+		}
+
+		if (BasisType::useSu2Symmetry())
+			err("SU(2) no longer supported\n");
 
 		ModelTermType& spsm = ModelBaseType::createTerm("SplusSminus");
 
 		OpForLinkType splus("splus");
 
-		auto valueModiferTerm0 = [isSu2](ComplexOrRealType& value)
-		{ value *= (isSu2) ? -0.5 : 0.5;};
+		auto valueModiferTerm0 = [](ComplexOrRealType& value) { value *= 0.5;};
 
 		typename ModelTermType::Su2Properties su2properties(2, -1, 2);
 		spsm.push(splus, 'N', splus, 'C', valueModiferTerm0, su2properties);
 
-		ModelTermType& szsz = ModelBaseType::createTerm("szsz");
-
-		if (!isSu2) {
-			OpForLinkType sz("sz");
-			szsz.push(sz, 'N', sz, 'N', typename ModelTermType::Su2Properties(2, 0.5));
-		} else {
-			auto valueModifierTermOther = [isSu2](ComplexOrRealType& value)
-			{ if (isSu2) value = -value;};
-			typename ModelTermType::Su2Properties su2properties(2, -1, 2);
-			spsm.push(splus, 'N', splus, 'C', valueModifierTermOther, su2properties);
-		}
+		connectionSzSz();
 
 		if (additional_ == "Anisotropic") {
 
@@ -359,6 +350,58 @@ protected:
 
 private:
 
+	void addMagneticField(SparseMatrixType& hmatrix,
+	                      char c,
+	                      SizeType site) const
+	{
+		assert(c == 'X' || c == 'Z');
+
+		const SizeType linSize = superGeometry_.numberOfSites();
+		const VectorRealType& v = (c == 'X') ? modelParameters_.magneticFieldX
+		                                     : modelParameters_.magneticFieldZ;
+
+		if (v.size() != linSize) return;
+
+		assert(site < v.size());
+		RealType tmp = v[site];
+		const OperatorType& sminus = ModelBaseType::naturalOperator("sminus", site, 0);
+
+		const OperatorType& sz = ModelBaseType::naturalOperator("sz", site, 0);
+
+		if (c == 'Z') {
+			hmatrix += tmp*sz.getCRS();
+			return;
+		}
+
+		assert(c == 'X');
+		const OperatorType& splus = ModelBaseType::naturalOperator("splus", site, 0);
+		constexpr RealType zeroPointFive = 0.5;
+		hmatrix += zeroPointFive*tmp*splus.getCRS();
+		hmatrix += zeroPointFive*tmp*sminus.getCRS();
+	}
+
+	void fillModelLinks2()
+	{
+		ModelTermType& sxsx = ModelBaseType::createTerm("sxsx");
+		OpForLinkType sx("sx");
+		sxsx.push(sx, 'N', sx, 'N');
+
+		auto sybarsybarModifier = [](ComplexOrRealType& value) { value *= -1.0;};
+		ModelTermType& sybarsybar = ModelBaseType::createTerm("sybarsybar");
+		OpForLinkType sybar("sybar");
+		sybarsybar.push(sybar, 'N', sybar, 'N', sybarsybarModifier);
+
+		connectionSzSz();
+	}
+
+	void connectionSzSz()
+	{
+		ModelTermType& szsz = ModelBaseType::createTerm("szsz");
+
+		OpForLinkType sz("sz");
+		szsz.push(sz, 'N', sz, 'N', typename ModelTermType::Su2Properties(2, 0.5));
+	}
+
 	//! Find S^+_site in the natural basis natBasis
 	SparseMatrixType findSplusMatrices(SizeType site,
 	                                   const HilbertBasisType& natBasis) const
@@ -460,16 +503,21 @@ private:
 	}
 
 
-	SparseMatrixType findSxMatrices(SizeType site,
-	                                const HilbertBasisType& natBasis) const
+	SparseMatrixType findSxOrSyBarMatrices(SizeType site,
+	                                       const HilbertBasisType& natBasis,
+	                                       PsimagLite::String what) const
 	{
+		if (what != "sx" && what != "sybar")
+			err("findSxOrSyBarMatrices: don't know how to calculate " + what + "\n");
+
+		const RealType sign = (what == "sx") ? 1.0 : -1.0;
 		SparseMatrixType Splus_temp=findSplusMatrices(site,natBasis);
 		SparseMatrixType Sminus_temp,Sx;
 		transposeConjugate(Sminus_temp,Splus_temp);
-		RealType tmp=0.5;
+		const RealType tmp = 0.5;
 
 		Sx = tmp*Splus_temp;
-		Sx += tmp*Sminus_temp;
+		Sx += sign*tmp*Sminus_temp;
 
 		return Sx;
 	}
@@ -490,9 +538,9 @@ private:
 			    ": Anisotropic sub-model CANNOT be canonical. Please " +
 			    "delete the TargetSzPlusConst= from the input file\n");
 
-		if (isCanonical && modelParameters_.magneticFieldDirection == "x")
+		if (isCanonical && !modelParameters_.magneticFieldX.empty())
 			err(PsimagLite::String(__FILE__) +
-			    ": magneticFieldDirection == x CANNOT be canonical. Please " +
+			    ": MagneticFieldX CANNOT be canonical. Please " +
 			    "delete the TargetSzPlusConst= from the input file\n");
 
 		VectorSizeType other;
@@ -526,7 +574,7 @@ private:
 		return sum;
 	}
 
-	ParametersModelHeisenberg<RealType, QnType>  modelParameters_;
+	ModelParametersType  modelParameters_;
 	const SuperGeometryType& superGeometry_;
 	SpinSquaredHelper<RealType,WordType> spinSquaredHelper_;
 	PsimagLite::String additional_;
