@@ -1,8 +1,8 @@
 /*
-Copyright (c) 2009-2013, UT-Battelle, LLC
+Copyright (c) 2009-2013-2021, UT-Battelle, LLC
 All rights reserved
 
-[DMRG++, Version 2.0.0]
+[DMRG++, Version 6.]
 [by G.A., Oak Ridge National Laboratory]
 
 UT Battelle Open Source Software License 11242008
@@ -88,7 +88,6 @@ DISCLOSED WOULD NOT INFRINGE PRIVATELY OWNED RIGHTS.
 #include "SpinSquared.h"
 #include "VerySparseMatrix.h"
 #include "ProgramGlobals.h"
-#include "MemResolv.h"
 
 namespace Dmrg {
 template<typename ModelBaseType> class ExtendedHubbard1Orb;
@@ -107,6 +106,7 @@ class ModelHubbard : public ModelBaseType {
 
 public:
 
+	typedef ModelHubbard<ModelBaseType> ThisType;
 	typedef unsigned int long WordType;
 	typedef  HilbertSpaceHubbard<WordType> HilbertSpaceHubbardType;
 	typedef typename ModelBaseType::ModelHelperType ModelHelperType;
@@ -145,6 +145,8 @@ public:
 	      spinSquared_(spinSquaredHelper_,NUMBER_OF_ORBITALS,DEGREES_OF_FREEDOM),
 	      extension_(extension)
 	{
+		ModelBaseType::onSiteHLegacyFix(modelParameters_.onSiteHaddLegacy);
+
 		SizeType usize = modelParameters_.hubbardU.size();
 		SizeType vsize = modelParameters_.potentialV.size();
 		SizeType totalSites = geometry.numberOfSites();
@@ -400,12 +402,14 @@ protected:
 	                                const BlockType& block,
 	                                RealType time)  const
 	{
+		ModelBaseType::additionalOnSiteHamiltonian(hmatrix, block, time);
+
 		SizeType n=block.size();
 		SparseMatrixType tmpMatrix,niup,nidown,Szsquare,Szi,Splusi,Sminusi;
 		SizeType linSize = ModelBaseType::superGeometry().numberOfSites();
 
 		for (SizeType i = 0; i < n; ++i) {
-			SizeType site = block[i];
+			const SizeType site = block[i];
 
 			// onsite U hubbard
 			//n_i up
@@ -449,19 +453,6 @@ protected:
 				hmatrix += tmp*Sminusi;
 				hmatrix += tmp*Splusi;
 			}
-
-			if (modelParameters_.potentialT.size()==0) continue;
-			RealType cosarg = cos(time*modelParameters_.omega +
-			                      modelParameters_.phase);
-			// VT_iup term
-			tmp = modelParameters_.potentialT[block[i]];
-			tmp *= cosarg;
-			hmatrix += tmp*niup;
-
-			// VT_idown term
-			tmp = modelParameters_.potentialT[block[i]];
-			tmp *= cosarg;
-			hmatrix += tmp*nidown;
 		}
 	}
 
